"""
Classifies: CHEBI:17478 aldehyde
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_aldehyde(smiles: str):
    """
    Determines if a molecule contains an aldehyde group (RC(=O)H).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains aldehyde group, False otherwise
        str: Reason for classification
    """
    try:
        mol = Chem.MolFromSmiles(smiles)
        if mol is None:
            return False, "Invalid SMILES string"
            
        # SMARTS patterns for aldehyde group
        # Pattern 1: Standard aldehyde [CH](=O)[#6]
        # Pattern 2: Explicit hydrogen notation [CH1](=O)[#6]
        # Pattern 3: Aldehyde with formal charge preservation [CH]([O])[#6]
        patterns = [
            '[CH](=O)[#6]',
            '[CH1](=O)[#6]',
            '[CH]([O])[#6]'
        ]
        
        total_matches = set()
        for pattern in patterns:
            pat = Chem.MolFromSmarts(pattern)
            matches = mol.GetSubstructMatches(pat)
            total_matches.update(matches)
            
        if len(total_matches) > 0:
            if len(total_matches) == 1:
                return True, "Contains 1 aldehyde group"
            else:
                return True, f"Contains {len(total_matches)} aldehyde groups"
                
        # Additional check for edge cases with alternative representations
        for atom in mol.GetAtoms():
            if (atom.GetAtomicNum() == 6 and  # Carbon
                atom.GetTotalNumHs() == 1 and  # One hydrogen
                atom.GetFormalCharge() == 0 and  # Neutral
                len([b for b in atom.GetBonds() if b.GetBondType() == Chem.BondType.DOUBLE]) == 1):  # One double bond
                
                # Check if double bonded to oxygen
                for bond in atom.GetBonds():
                    if (bond.GetBondType() == Chem.BondType.DOUBLE and 
                        mol.GetAtomWithIdx(bond.GetOtherAtomIdx(atom.GetIdx())).GetAtomicNum() == 8):
                        return True, "Contains 1 aldehyde group"
                
        return False, "No aldehyde groups found"
        
    except Exception as e:
        return None, f"Error processing molecule: {str(e)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17478',
                          'name': 'aldehyde',
                          'definition': 'A compound RC(=O)H, in which a '
                                        'carbonyl group is bonded to one '
                                        'hydrogen atom and to one R group.',
                          'parents': ['CHEBI:36586']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.6578073089700996 is too low.\n'
               "True positives: [('CC1C(C=O)=CNC=C1C=O', 'Contains 2 aldehyde "
               "groups'), ('O=CC1=C(O)C=CC(=C1CC[C@@H](O)[C@H](O)C)CC=C(C)C', "
               "'Contains 1 aldehyde group'), ('O=CCCC/C=C/C=C/C', 'Contains 1 "
               "aldehyde group'), "
               "('O=C/C(=C/CCCCCCCCCCCCCC)/CCCCC/C=C\\\\C/C=C\\\\CCCCC', "
               "'Contains 1 aldehyde group'), "
               "('OC(=O)c1cc(O)c(C(=O)c2c(O)cccc2O)c(C=O)c1', 'Contains 1 "
               "aldehyde group'), "
               "('O=C1[C@@H]([C@@](/C=C/C(=C/CC2=C(O)C(=C(C)C=C2O)C=O)/C)([C@H](C)CC1)C)C', "
               "'Contains 1 aldehyde group'), ('C(C=1C=CC(=CC1)N(C)C)(=O)[H]', "
               "'Contains 1 aldehyde group'), ('[H]C(=O)CCC(O)=O', 'Contains 1 "
               "aldehyde group'), ('O=CCCC=C', 'Contains 1 aldehyde group'), "
               "('[H]C(=O)\\\\C=C\\\\C(C(O)=O)=C(\\\\O)C(O)=O', 'Contains 1 "
               "aldehyde group'), ('[H]C(=O)C1CCCCC1', 'Contains 1 aldehyde "
               "group'), ('O=CC(CCCCCCCCC)C', 'Contains 1 aldehyde group'), "
               "('C12C(C3C(C(CC3)*)(C)CC1)CCC4C2(CCC[C@@H]4C=O)C', 'Contains 1 "
               "aldehyde group'), ('O=C/C(=C/C=1N=COC1)/[C@@H](O)C', 'Contains "
               "1 aldehyde group'), ('O=CCC/C=C/CCC=O', 'Contains 2 aldehyde "
               "groups'), ('O(C=1C(=CC(OC)=C(OC)C1)/C=C\\\\C=O)C', 'Contains 1 "
               "aldehyde group'), ('[H]C(=O)c1ccc(CO)o1', 'Contains 1 aldehyde "
               "group'), "
               "('C[C@@H]([C@H]1CC[C@H]2[C@@H]3CCC4=CC(=O)C=C[C@]4(C)[C@H]3CC[C@]12C=O)[C@H]1CC(C)=C(C)C(=O)O1', "
               "'Contains 1 aldehyde group'), ('S(C(CC=O)C)C', 'Contains 1 "
               "aldehyde group'), "
               "('S(=O)(=O)(O[C@H](CC=CC1=C(C(O)=CC=C1)C=O)CC)O', 'Contains 1 "
               "aldehyde group'), "
               "('COc1cc(C=O)cc2[C@@H](CO)[C@@H](Oc12)c1cc(OC)c(O[C@H](CO)[C@@H](O)c2ccc(O)c(OC)c2)c(OC)c1', "
               "'Contains 1 aldehyde group'), "
               "('O[C@@H](CCCC(O)=O)\\\\C=C/C=C/C=C/[C@H](O)C\\\\C=C/CCCCC=O', "
               "'Contains 1 aldehyde group'), ('O=C(CCCCN)[H]', 'Contains 1 "
               "aldehyde group'), ('OC(=O)c1ccc(C=O)cc1O', 'Contains 1 "
               "aldehyde group'), ('O=CC(CCCC)CC', 'Contains 1 aldehyde "
               "group'), ('CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC=O', 'Contains 1 "
               "aldehyde group'), ('OCC(CC=O)C(O)=O', 'Contains 1 aldehyde "
               "group'), ('O=CCC/C=C/C=C', 'Contains 1 aldehyde group'), "
               "('[H]C(=O)[C@@H](O)[C@@H](O)CC(=O)C(O)=O', 'Contains 1 "
               "aldehyde group'), ('O=C(NCCCCCN1C(=CC=C1)C=O)C', 'Contains 1 "
               "aldehyde group'), "
               "('[H][C@@]12CCC(C)=C[C@]1([H])[C@@]([H])(CC[C@H]2C)[C@@H](C)C=O', "
               "'Contains 1 aldehyde group'), "
               "('CC1(C)CC[C@@]2(CC[C@]3(C)C(C=C[C@@H]4[C@@]5(C)CC[C@@H](O)[C@@](C)(C=O)[C@@H]5CC[C@@]34C)=C2C1)C(O)=O', "
               "'Contains 1 aldehyde group'), ('C(/C=C\\\\CCC)=O', 'Contains 1 "
               "aldehyde group'), ('[H]C(=O)\\\\C=C\\\\C([H])=O', 'Contains 2 "
               "aldehyde groups'), "
               "('CC12CCC3C(C1(CCC2C4=CC(=O)OC4)O)CCC5(C3(CCC(C5)O)C=O)O', "
               "'Contains 1 aldehyde group'), "
               "('O=C1[C@@H]([C@]([C@H](C)CC1)(C[C@@H](OC(=O)C)/C(=C/CC2=C(O)C(=C(C)C=C2O)C=O)/C)C)C', "
               "'Contains 1 aldehyde group'), "
               "('ClC1=C(O)C(=C(O)C(=C1C)C=O)C/C=C(/[C@H](OC(=O)CCC)C[C@]2([C@@H](C(=O)CC[C@@H]2C)C)C)\\\\C', "
               "'Contains 1 aldehyde group'), ('[H]C(=O)CCCCCCCCCCCCCCC', "
               "'Contains 1 aldehyde group'), "
               "('C\\\\C=C1\\\\CN2[C@H]3Cc4c([nH]c5ccccc45)[C@@H]2C[C@@H]1[C@@H]3C=O', "
               "'Contains 1 aldehyde group'), ('O=CCCCCCCCCC/C=C\\\\CC', "
               "'Contains 1 aldehyde group'), "
               "('CC(CC/C=C(\\\\CC/C=C(\\\\CCC(=O)[H])/C)/C)=O', 'Contains 1 "
               "aldehyde group'), ('OC(=O)CCCCCCCCCCCCCCCCCCCCC=O', 'Contains "
               "1 aldehyde group'), ('O=C\\\\C=C\\\\C=C\\\\C/C=C\\\\CCCCC', "
               "'Contains 1 aldehyde group'), ('[H]C(=O)C#C[*]', 'Contains 1 "
               "aldehyde group'), ('P(OCC=1C(=C(O)C(=NC1)C)C=O)(O)(O)=O.O', "
               "'Contains 1 aldehyde group'), "
               "('[H][C@@]1(CC[C@H](C)[C@@]1([H])C=O)C(=C)C=O', 'Contains 2 "
               "aldehyde groups'), ('O=CCC/C=C\\\\CC', 'Contains 1 aldehyde "
               "group'), ('O=CCCCCCCCCCC/C=C\\\\CCC', 'Contains 1 aldehyde "
               "group'), ('N1=C(C2=NC=CC(=C2)C)C=C(C=C1)C([H])=O', 'Contains 1 "
               "aldehyde group'), ('O=CCCCC/C=C/C=C/C=C/C=C/CC', 'Contains 1 "
               "aldehyde group'), "
               "('O=C/C(=C/CCCCCCCCCCCCCC)/CCCCC/C=C\\\\CCCCCCCC', 'Contains 1 "
               "aldehyde group'), ('[H]C(=O)CCCCCCC([H])=C([H])C', 'Contains 1 "
               "aldehyde group'), ('O=CCCC=C=C', 'Contains 1 aldehyde group'), "
               "('[H][C@@]12CC(C)(C)CC[C@@]1(CC[C@]1(C)C2=CC[C@]2([H])[C@@]3(C)CC[C@H](O)[C@@](C)(C=O)[C@]3([H])CC[C@@]12C)C(O)=O', "
               "'Contains 1 aldehyde group'), ('O=C(/C=C/C(=O)C=O)C', "
               "'Contains 1 aldehyde group'), ('C=C(C)C=O', 'Contains 1 "
               "aldehyde group'), "
               "('C1=C(C=C2C(=C1OC)O[C@H]([C@@H]2C=O)C3=CC=C(C(=C3)OC)O)/C=C/C(O)=O', "
               "'Contains 1 aldehyde group'), "
               "('O=CCC\\\\C=C\\\\C=C/CCCCCCCCC', 'Contains 1 aldehyde "
               "group'), ('CCCC(C)CC=O', 'Contains 1 aldehyde group'), "
               "('C12=NC=C(N=C2C(=NC(=N1)N)N)C=O', 'Contains 1 aldehyde "
               "group'), "
               "('[N+]([O-])(=O)C1=C2C(=C3C(=C1)C(OC)=CC=C3)C=4OCOC4C=C2C(O[C@@H]5CC(=CCCC(=CCCC(=C5)C([H])=O)C)C)=O', "
               "'Contains 1 aldehyde group'), ('CC[C@H](C)C(=O)[H]', 'Contains "
               "1 aldehyde group'), "
               "('CC(=O)O[C@H]1CC[C@]2(C=O)[C@H]3CC[C@@]4(C)[C@@H](CCC4=O)[C@@H]3CC=C2C1', "
               "'Contains 1 aldehyde group'), "
               "('O=C/C=C/C=C/C=C/C=C/C=C/C=C/C=C/C=C/C=C/C', 'Contains 1 "
               "aldehyde group'), "
               "('[H]C(=O)\\\\C=C(/C)CC\\\\C=C(/C)CCC=C(C)C', 'Contains 1 "
               "aldehyde group'), ('O=CC=1N(C=CC1)C', 'Contains 1 aldehyde "
               "group'), "
               "('C[C@@H]1C[C@@H](OC(C)=O)[C@H](O)[C@H](O[C@H]2CC[C@]3(C=O)[C@H]4[C@H](O)C(=O)[C@]5(C)[C@H](CC[C@]5(O)[C@@H]4CC[C@]3(O)C2)c2ccc(=O)oc2)O1', "
               "'Contains 1 aldehyde group'), "
               "('CC(C)=CCc1c(O)cc2Oc3c(C=O)c(O)c(Cl)c(C)c3C(=O)Oc2c1C', "
               "'Contains 1 aldehyde group'), ('O=CCC(CC(C)(C)C)C', 'Contains "
               "1 aldehyde group'), "
               "('[C@H]1(O[C@@H]([C@@H](O)[C@@H]([C@H]1O)O)CO)O[C@H]2[C@@H]([C@@H](C(=CO2)C(OC)=O)CC=O)C=C', "
               "'Contains 1 aldehyde group'), "
               "('CC(C)[C@@H]1C[C@@H](O)[C@H]2[C@]1(CC[C@@]1(C)[C@@H]3[C@@H](O)C[C@H]4C(C)(C)[C@H](CC[C@]4(C)C3=CC[C@]21C)O[C@@H]1O[C@H](CO[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)[C@@H](O)[C@H](O)[C@H]1O)C=O', "
               "'Contains 1 aldehyde group'), "
               "('[H]C(=O)[C@H]1C(=CC[C@@]2([H])[C@]3(C)CC[C@@]4([H])C(C)(C)CCC[C@]4(C)[C@@]3([H])C[C@H](OC(C)=O)[C@]12C)C([H])=O', "
               "'Contains 2 aldehyde groups'), ('O=Cc1ccco1', 'Contains 1 "
               "aldehyde group'), "
               "('CC1(C)CC[C@]2(C=O)[C@H](O)C[C@]3(C)[C@@H]([C@@H]2C1)C(=O)C[C@@H]1[C@@]2(C)CC[C@H](O[C@@H]4OC[C@H](O[C@@H]5O[C@H](CO)[C@@H](O)[C@H](O)[C@H]5O)[C@H](O)[C@H]4O[C@@H]4O[C@H](CO)[C@@H](O)[C@H](O)[C@H]4O)C(C)(C)[C@@H]2CC[C@@]31C', "
               "'Contains 1 aldehyde group'), "
               "('[H][C@@]12C[C@@]3([H])C(C)(C)O[C@@](C\\\\C=C(\\\\C)C=O)(C1=O)[C@]31Oc3c(CC=C(C)C)c4OC(C)(C)C=Cc4c(O)c3C(=O)C1=C2', "
               "'Contains 1 aldehyde group'), "
               "('[H][C@]12[C@H](O)[C@]34C[C@@H](C[C@@H](OC(C)=O)[C@@]3([H])[C@@]1(CCCC2(C)C)COC4=O)C(=C)C=O', "
               "'Contains 1 aldehyde group'), ('CC(C=O)C1CCC(C)=CC1', "
               "'Contains 1 aldehyde group'), "
               "('N[C@@H]1C[C@H](N)[C@@H](O[C@H]2O[C@H](C=O)[C@@H](O)[C@H](O)[C@H]2N)[C@H](O)[C@H]1O', "
               "'Contains 1 aldehyde group'), "
               "('Cc1c(Cl)c(O)c(C=O)c2Oc3cc4OC(C)(C)CC(=O)c4c(C)c3OC(=O)c12', "
               "'Contains 1 aldehyde group'), ('O=C\\\\C=C\\\\CCCCCCCCCCC', "
               "'Contains 1 aldehyde group'), "
               "('N[C@@H]1C[C@H](N)[C@@H](O[C@H]2O[C@H](C=O)[C@@H](O)[C@H](O)[C@H]2O)[C@H](O)[C@H]1O', "
               "'Contains 1 aldehyde group'), ('CC(=O)\\\\C=C\\\\C=C(/C)C=O', "
               "'Contains 1 aldehyde group'), ('[H]C(=O)C(O)CCCCCCCCCCCCCCCC', "
               "'Contains 1 aldehyde group'), "
               "('[H][C@@]12C[C@H](O)CC[C@]1(C)[C@@]1([H])C[C@H](O)[C@]3(C)[C@]([H])(CC[C@@]3([H])[C@]1([H])[C@H](O)C2)[C@H](C)CCC(O)C(C)C=O', "
               "'Contains 1 aldehyde group'), ('[H]C(=O)CCCCCCCCC(O)=O', "
               "'Contains 1 aldehyde group'), ('O=CCCCCCC', 'Contains 1 "
               "aldehyde group'), "
               "('[H][C@@]12CCC3=C(CC[C@]4(C)[C@]([H])(CC[C@@]34[H])[C@H](C)CCC=C(C)C)[C@@]1(C)CC[C@H](O)C2C=O', "
               "'Contains 1 aldehyde group'), ('O1C(C1CC=O)CCCCC', 'Contains 1 "
               "aldehyde group'), ('O=C\\\\C=C\\\\C=C\\\\C=C\\\\C=C\\\\C', "
               "'Contains 1 aldehyde group'), ('OC=1C=C(NC1C=O)C=2NC=CC2', "
               "'Contains 1 aldehyde group'), ('O1C(C1C=CC=O)CCCCC', 'Contains "
               "1 aldehyde group'), ('[H]C(=O)C([H])=C([H])CCC', 'Contains 1 "
               "aldehyde group'), ('O=C/C=C\\\\CCCCCCC', 'Contains 1 aldehyde "
               "group'), ('O=CCCCCCCCCC/C=C/CC', 'Contains 1 aldehyde group'), "
               "('[H]C(=O)[C@@H](N)Cc1c[nH]cn1', 'Contains 1 aldehyde group'), "
               "('CC(C)=CC(=O)c1c(O)cc2Oc3c(C=O)c(O)cc(C)c3C(=O)Oc2c1C', "
               "'Contains 1 aldehyde group'), "
               "('[H]C(=O)C(C)CCC[C@@H](C)[C@@]1([H])CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@H](O)CC[C@]4(C)[C@@]3([H])C[C@H](O)[C@]12C', "
               "'Contains 1 aldehyde group'), "
               "('C=1(C=C(C(=C(C1C)O)C=O)CC(CCCCCCCCC)=O)O', 'Contains 1 "
               "aldehyde group'), ('OC1=C(C(C)(C)C)C=C(C=C1C(C)(C)C)C=O', "
               "'Contains 1 aldehyde group')]\n"
               'False positives: '
               "[('O=C1OC2=C(C(C(=O)O)=CC(=C2)OC)OC3=C1C(=CC(=C3C=O)O)C', "
               "'Contains 1 aldehyde group'), "
               "('O=C(C1=C2O[C@]3(OC[C@@H]([C@H]3CC2=C(C)C=C1O)C)C)C4=C(C(OC)=CC(=C4)O)C=O', "
               "'Contains 1 aldehyde group'), "
               "('OC1=C(C(=C(CC/C(=C/CC2=C(O)C=CC(O)=C2)/C)C(=C1)C)C)C=O', "
               "'Contains 1 aldehyde group'), "
               "('O=C1C=CC(=CC([C@@H](OC(=O)C[C@H]([C@@H]([C@H]([C@H](C[C@H]1C)CC=O)OC2OC(C(OC3OC(C(OC(=O)CC(C)C)C(C3)(O)C)C)C(C2O)N(C)C)C)C)OC(=O)C)CC)COC4OC(C(O)C(C4OC)OC)C)C', "
               "'Contains 1 aldehyde group'), "
               "('O1C(CCC=2C1=C(C(=CC2OC)COC(=O)CCCCCCCCCCCCCCC)C=O)(CC(=O)C=C(C)C)C', "
               "'Contains 1 aldehyde group'), "
               "('[H]C([H])([C@@]([H])(O)C=O)[C@@]([H])(O)[C@@]([H])(C)O', "
               "'Contains 1 aldehyde group'), "
               "('CC(C)=CCc1c(O)c(C=O)cc2c3ccccc3[nH]c12', 'Contains 1 "
               "aldehyde group'), ('O=CC1=C2C(OC(=C2)C)=CC(=C1)O', 'Contains 1 "
               "aldehyde group'), ('S(CC1=C(OC)C=CC(=C1)C=O)CC=2OC=CC2', "
               "'Contains 1 aldehyde group'), "
               "('O=CC1=C(OC)C2=C(C=C(C)OC2)C=C1O', 'Contains 1 aldehyde "
               "group'), "
               "('O=C1C=C(C)[C@@H]2[C@H]1C(=CC[C@H]3[C@@H]([C@H](/C=C\\\\[C@H](O)C(O)(C)C)C)CC[C@@]3(C2)C)C=O', "
               "'Contains 1 aldehyde group'), "
               "('CC(=O)O[C@H]1CC[C@]2(C=O)[C@H]3CC[C@]4(C)[C@H](CC[C@]4(O)[C@@H]3CC[C@]2(O)C1)c1ccc(=O)oc1', "
               "'Contains 1 aldehyde group'), "
               "('ON1CC(CC2=CC=CC=C2)=C(C=C1)C=O', 'Contains 1 aldehyde "
               "group'), ('OC(=O)CCC/C=C\\\\CC/C=C/C=O', 'Contains 1 aldehyde "
               "group'), ('[H]C(=O)[C@@H](N)CCC([O-])=O', 'Contains 1 aldehyde "
               "group'), ('O=Cc1c2ccc(cc3ccc(cc4ccc(cc5ccc1[nH]5)n4)[nH]3)n2', "
               "'Contains 1 aldehyde group'), "
               "('O=C1OC2=C(C(=C(O)C(=C2C)C(=O)O)CO)OC3=C1C(=CC(=C3C=O)O)C', "
               "'Contains 1 aldehyde group'), ('OC(=O)CCCCCC=O', 'Contains 1 "
               "aldehyde group'), "
               "('OC1C2C3(C(C(CC3)C(CC(O)C=C(C)C)C)(CCC2(C4C(=C1)C(C(O)CC4)(C)C)C=O)C)C', "
               "'Contains 1 aldehyde group'), "
               "('[H]C(=O)C\\\\C=C/CC(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12', "
               "'Contains 1 aldehyde group'), "
               "('[H][C@@]12CC[C@@]3([H])[C@@]4(CC[C@H](O)[C@@](C)(COC(C)=O)[C@H]4C=O)C(OC(=O)[C@]3(C1)C(=O)C2=C)C(C)=O', "
               "'Contains 1 aldehyde group'), "
               "('O=C1[C@]2([C@@H](C3=C([C@H](CO)C)CC[C@]3(CO)CC2)CC=C(C1)C=O)C', "
               "'Contains 1 aldehyde group'), "
               "('ClC/C(=C\\\\CCl)/CC/C=C(/C=O)\\\\C', 'Contains 1 aldehyde "
               "group'), "
               "('[H]C(=O)CCCC(=O)O[C@H](COC(=O)CCCCC)COP(O)(=O)OCC[N+](C)(C)C', "
               "'Contains 1 aldehyde group'), "
               "('C[C@@]1(CC[C@]23CO[C@@]4(CC[C@@H]5[C@@]6(C)CC[C@H](O[C@@H]7OC[C@H](O[C@@H]8O[C@H](CO)[C@@H](O)[C@H](O[C@@H]9O[C@H](CO)[C@@H](O)[C@H](O)[C@H]9O)[C@H]8O[C@@H]8OC[C@@H](O)[C@H](O)[C@H]8O)[C@H](O)[C@H]7O[C@@H]7O[C@H](CO)[C@@H](O)[C@H](O)[C@H]7O)C(C)(C)[C@@H]6CC[C@@]5(C)[C@]4(C)C[C@H]2O)[C@@H]3C1)C=O', "
               "'Contains 1 aldehyde group'), ('[H]C(=O)CP(O)([O-])=O', "
               "'Contains 1 aldehyde group'), "
               "('CO[C@H]1C[C@H](O[C@H]2CC[C@]3(C=O)[C@H]4CC[C@]5(C)[C@H](CC[C@]5(O)[C@@H]4CC[C@]3(O)C2)C2=CC(=O)OC2)O[C@H](C)[C@H]1O', "
               "'Contains 1 aldehyde group'), "
               "('CC(C)C[C@@H]1[C@H]2CC(C=C[C@@]2(C)Oc2c(C=O)c(O)c(C=O)c(O)c12)C(C)C', "
               "'Contains 2 aldehyde groups'), "
               "('O=C(OC1=C(C(O)=C(C(=O)O)C(=C1)C)C)C2=C(O)C(=C(OC)C=C2C)C=O', "
               "'Contains 1 aldehyde group'), "
               "('O=C(NC(C(=O)NC(C(=O)NC(C=O)C)CCC(=O)N)C1NC(=NCC1)N)NC(C(=O)O)CC(C)C', "
               "'Contains 1 aldehyde group'), "
               "('O1[C@@]23[C@@]([C@H]([C@@H](O)C12)C=4C=CC(OC4)=O)(CC[C@H]5[C@H]3CC[C@H]6[C@@]5(CC[C@H](O)C6)C=O)C', "
               "'Contains 1 aldehyde group'), "
               "('O=C(OC)CC(NC1=CC=C(C=O)C=C1)C(O)/C=C/C(O)C', 'Contains 1 "
               "aldehyde group'), "
               "('O=C(NC(C=O)CCCN=C(N)N)C1N(C(=O)C(NC(=O)C(OC(=O)C)CC2=CC=C(O)C=C2)CC(C)C)CCC1', "
               "'Contains 1 aldehyde group'), "
               "('O=C1O[C@H]([C@@H](O)CC)C[C@H]1C/C=C/C=O', 'Contains 1 "
               "aldehyde group'), "
               "('CCC1=C(C=O)C2=[N+]3C1=Cc1c(C)c4C(=O)[C-](C(=O)OC)C5=C6[C@@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)[C@H](C)C7=[N+]6[Mg--]3(n1c45)n1c(=C7)c(C)c(C=C)c1=C2', "
               "'Contains 1 aldehyde group'), "
               "('O=C1OC(=C)[C@H](C)C=2C1=C(O)C(=C(O)C2C=O)C', 'Contains 1 "
               "aldehyde group'), ('O=CC1=C(O)C=CC(=C1/C=C/CCCCC)OC', "
               "'Contains 1 aldehyde group'), ('O=C1C=2C(NC(=C1)C=O)=CC=CC2', "
               "'Contains 1 aldehyde group'), "
               "('[C@H]1(O[C@@](C[C@@H]([C@H]1NC(=O)C)O)(OCC2[C@@H]([C@@H]([C@H]([C@@H](O2)O[C@@H]3[C@H](O[C@H]([C@@H]([C@H]3O)NC(=O)C)O[C@@H]4[C@H]([C@@H](O[C@@H]([C@@H]4O)CO)O[C@H]([C@H](O)CO)[C@@H]([C@@H](O)C=O)O)O)CO)O)O)O)C(=O)O)[C@@H]([C@@H](CO)O)O', "
               "'Contains 1 aldehyde group'), "
               "('O=C1N(C(=O)C2=CC=3[C@H](N2C1=O)[C@@H](OC(=O)C4=CC(OC5=C(OC)C=CC(=C5)C=O)=C(OC)C=C4)C=COC3)C', "
               "'Contains 1 aldehyde group'), ('O(C=1C(=CC=CC1)C=O)C', "
               "'Contains 1 aldehyde group'), "
               "('SC[C@H](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)[C@@H](NC(=O)CNC(=O)[C@@H](NC(=O)CNC(=O)CNC(=O)[C@@H](NC(=O)CNC(=O)[C@@H](NC(=O)[C@@H](N)CCCCN)CC(=O)N)CC1=CC=C(O)C=C1)CO)CC(=O)N)CCCNC(=N)N)CC=2C3=C(C=CC=C3)NC2)C(C)C)CC=4NC=NC4)C(=O)NCC(=O)N[C@H](C(=O)NCC(=O)N[C@H](C(=O)N[C@H](C(=O)NCC=O)C(C)C)[C@H](CC)C)C', "
               "'Contains 1 aldehyde group'), "
               "('ClC1=C(O)C=C(O)C(=C1C)C(=O)O[C@H]2[C@@]3(O)C(=C[C@@H]4CC([C@@H]([C@@H]4[C@@]3(C)C2)O)(C)C)C=O', "
               "'Contains 1 aldehyde group'), "
               "('[H]C(=O)CCC[C@H]([NH3+])C([O-])=O', 'Contains 1 aldehyde "
               "group'), ('Oc1cc([nH]c(=O)c1CCC=O)C([O-])=O', 'Contains 1 "
               "aldehyde group'), "
               "('O[C@@]12C3C([C@@]4([C@](O)(CC3)C[C@@H](OC5OC(C(OC(=O)C)C(OC)C5O)C)CC4)C=O)C(OC(=O)C)C[C@@]1([C@H](CC2)C=6COC(=O)C6)C', "
               "'Contains 1 aldehyde group'), ('O=CC1C(CCC1C(C)=C)C', "
               "'Contains 1 aldehyde group'), "
               "('[H][C@@](O)(CO)[C@]([H])(O)[C@]([H])(O)[C@@]([H])(O[C@@H]1O[C@@H](C)[C@@H](O)[C@@H](O)[C@@H]1O)C=O', "
               "'Contains 1 aldehyde group'), ('OCC(O)(CO)[C@@H](O)C=O', "
               "'Contains 1 aldehyde group'), "
               "('[C@H]1([C@@H](O[C@@H]([C@H]([C@@H]1O)O)O[C@H]2[C@@H](O[C@@H]([C@@H]([C@@H]2O)O)CO)OC3[C@H]([C@@H](O[C@@H]([C@H]3O)CO)O[C@H]4[C@H]([C@H](O[C@H]([C@@H]4C)OC([C@@H]([C@@H](O)C=O)O)[C@@H](CO)O)CO[C@@H]5O[C@@H]([C@H]([C@@H]([C@H]5NC(=O)C)O[C@@H]6O[C@H]([C@H]([C@H]([C@@H]6O)O)O)O)O[C@@H]7[C@@H]([C@H]([C@H]([C@H](O7)CO)O)O)O)CO)O)NC(=O)C)C)O', "
               "'Contains 1 aldehyde group'), "
               "('O=CC1=C2[C@@](CC([C@H]2[C@@H](O)C[C@H]1C)(C)C)(CO)C', "
               "'Contains 1 aldehyde group'), "
               "('O=CC1=C(C=C(O)C2=C1O[C@]3([C@@]4([C@H](C([C@H](O)[C@@H](C4)O)(C)C)CC[C@H]3C)C)C2)CO', "
               "'Contains 1 aldehyde group'), "
               "('S(OC)(=O)C=1NC=2C(C1C=O)=CC=CC2', 'Contains 1 aldehyde "
               "group'), "
               "('O([C@@H]1[C@]2([C@]3([C@@]([C@](CC3)([C@@H](C/C=C/C(O)(C)C)C)[H])(CC[C@]2([C@]4(C(=C1)C([C@@H](O)CC4)(C)C)[H])C=O)C)C)[H])[C@@H]5O[C@@H]([C@@H](O)[C@H](O)[C@H]5O)CO', "
               "'Contains 1 aldehyde group'), "
               "('O=C1C(\\\\C(=C/2\\\\N=C(C(C2CCC(O)=O)C)CC3NC(=O)C(=C3C)C=C)\\\\C=4NC(=C(C41)C)CC=5NC(=C(C5CC)C)C=O)C(OC)=O', "
               "'Contains 1 aldehyde group'), "
               "('ClC1=C(O)C(=C(OC)C=C1C)C(=O)C2=C(C(O)=CC(=C2CC=C(C)C)O)C=O', "
               "'Contains 1 aldehyde group'), "
               "('O=C(O[C@@H]1[C@H](OC)[C@H](O[C@H]([C@H]1O)OC[C@]23[C@@]4(C(C(C)C)=C[C@@H]2C[C@]4(C=O)[C@@H]5CC[C@H]([C@H]5C3)C)C(=O)O)C)/C(=C/C=C\\\\C)/C', "
               "'Contains 1 aldehyde group'), "
               "('C=C[C@H]1CN2CCc3c([nH]c4ccccc34)[C@@H]2C[C@@H]1CC=O', "
               "'Contains 1 aldehyde group'), ('OC(=O)C(=O)CC=O', 'Contains 1 "
               "aldehyde group'), "
               "('O=CC1=C(O)C(=CC2=C1CC[C@@H](O2)[C@H](O)C=CC)CC=C(C)C', "
               "'Contains 1 aldehyde group'), "
               "('P(OC[C@H](OC(=O)CCC(O)/C=C/C=O)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Contains 1 aldehyde group'), "
               "('C1CN(CCN1C2=CC=C(C=C2)C=O)[S+](=O)(C3=CC4=C(C=C3)NC(=O)C4)[O-]', "
               "'Contains 1 aldehyde group'), "
               "('O([C@H]1[C@@H](OC2O[C@H]([C@@H](O)[C@@H](O)[C@@H]2O)C)[C@H](C[C@@H](O[C@@H]([C@H](O[C@@H]3O[C@H]([C@@H](O)[C@@H](O)[C@@H]3O)C)[C@@H](O)C=O)[C@H](O)CO)[C@@H]1O)CO)[C@@H]4O[C@@H]([C@@H](O)[C@H](O[C@@H]5O[C@@H]([C@@H](O)[C@H](O)[C@H]5O)CO)[C@H]4NC(=O)C)CO', "
               "'Contains 1 aldehyde group'), "
               "('[H][C@@](C)(O)[C@]([H])(N)[C@@](C)(O)[C@@]([H])(OC)C=O', "
               "'Contains 1 aldehyde group'), "
               "('O=C(O[C@@H]1C=C(C=O)[C@](O)(C=O)[C@@]2([C@@H]1C(CCC2)(C)C)C)/C=C/C=C/C=C/C', "
               "'Contains 2 aldehyde groups'), "
               "('O=CC=1C2=C([C@H]([C@@H](O)C[C@H]2O)C)N(C1)C', 'Contains 1 "
               "aldehyde group'), ('OC(=O)CCC/C=C\\\\C/C=C\\\\CC=O', 'Contains "
               "1 aldehyde group'), "
               "('CC[C@@]1(CN2CCC=3C4=CC=CC=C4NC3[C@@]2(C[C@@]1(CC=O)[H])[H])[H]', "
               "'Contains 1 aldehyde group'), "
               "('O=C1C23OC(C1C(O)C(=CC=CC4(C=C(C(C)CC45OC([O-])C(C5=O)C(O)C=CC6(C=C(C=O)C(C)CC67OC(C(C(C(=CC=CC8C9(C(C(C(C=CC2(C=C(C=O)C(C3)C)C)O)C(O9)[O-])=O)CC(C)C(=C8)CO[C@H]%10O[C@@H]([C@H](O)[C@@H]([C@H]%10O)O)CO)C)O)C7=O)[O-])C)CO[C@H]%11O[C@@H]([C@H](O)[C@@H]([C@H]%11O)O)CO)C)C)[O-]', "
               "'Contains 2 aldehyde groups'), "
               "('O=C1O[C@H](O)[C@@H]2[C@]13[C@@H](O)[C@@H](OC(=O)[C@H](O)CCCCCCCC)CC([C@@H]3CC=C2C=O)(C)C', "
               "'Contains 1 aldehyde group'), "
               "('O=C1N(C(=O)C2=CC=3[C@H](N2C1=O)[C@@H](OC(=O)C4=CC(OC5=C(O)C=CC(=C5)C=O)=C(OC)C=C4)C=COC3)C', "
               "'Contains 1 aldehyde group'), "
               "('S1CC(O)(N2C1=C(C=3C2=CC=CC3)C=O)C', 'Contains 1 aldehyde "
               "group'), "
               "('O=C1OC(CC2OC2C=CC(O)C(CC(C(C(C(C1)OC(=O)CC)OC)OC3OC(C(OC4OC(C(OC(=O)CC(C)C)C(C4)(O)C)C)C(C3O)N(C)C)C)CC=O)C)C', "
               "'Contains 1 aldehyde group'), "
               "('CC(C)(C)C1=CC(=CC(=C1)C=O)C(C)(C)C', 'Contains 1 aldehyde "
               "group'), ('O=C(O)C(=O)C=1C(=C[C@@H]2CC(C[C@@H]2C1C)(C)C)C=O', "
               "'Contains 1 aldehyde group'), "
               "('CCC(=O)O[C@H]1[C@H](C)O[C@H](C[C@@]1(C)O)O[C@@H]1[C@@H](C)O[C@@H](O[C@H]2[C@@H](CC=O)C[C@@H](C)[C@@H](O)\\\\C=C\\\\C=C\\\\C[C@@H](C)OC(=O)C[C@@H](OC(C)=O)[C@@H]2OC)[C@H](O)[C@H]1N(C)C', "
               "'Contains 1 aldehyde group'), "
               "('O=CC1=C(O)C=CC=C1C=CCCC[C@@H](O)C', 'Contains 1 aldehyde "
               "group'), ('O(C1C(C2C(C(C(O)(CC2)C)C=O)(CC1)C)(C)C)C(=O)C', "
               "'Contains 1 aldehyde group'), ('Cl\\\\C=C/C=O', 'Contains 1 "
               "aldehyde group'), "
               "('O=C[C@]12[C@@H](O[C@H](C(O)(C)C)C[C@H]1O)CC[C@]3([C@H]2CC[C@@H]4[C@@]3(C=5NC=6C=CC=CC6C5C4)C)C', "
               "'Contains 1 aldehyde group'), "
               "('O=CC1=C(O)C=CC2=C1[C@@](O)(C3=C4O[C@H](C(O)(C)C)COC4=CC(=C3)C)[C@H](C(=C)C)C2', "
               "'Contains 1 aldehyde group'), "
               "('O=C(O[C@H]1[C@H](O)C([C@@H]2CC[C@H]([C@]3([C@]2(C1)C)OC=4C=C(CO)C(=C(C4C3)O)C=O)C)(C)C)C', "
               "'Contains 1 aldehyde group'), ('O=CC1(C2(C3C2CC1C3)C)C', "
               "'Contains 1 aldehyde group'), "
               "('O=C(NC(C=O)CCCN=C(N)N)[C@H](NC(=O)C(NC(=O)C(O)CC1=CC=C(O)C=C1)CCC2=CC=CC=C2)[C@@H](CC)C', "
               "'Contains 1 aldehyde group'), "
               "('O=CC1=CCC2C3=C(C(C)C)CC[C@]3(C)CC[C@]2([C@H]([C@H]1OC)O)C', "
               "'Contains 1 aldehyde group'), "
               "('O1C(OC2=C(CCC=O)C=CC=C2O)C(O)C(O)C(O)C1C(O)=O', 'Contains 1 "
               "aldehyde group'), ('O=C(O)[C@H](N)C1=CC(O)=C(C=O)C=C1', "
               "'Contains 1 aldehyde group'), "
               "('O=[N+]([O-])[C@@]1([C@H](NC(=O)OC)[C@H](O[C@H](C1)O[C@H]2C(=C[C@@H]3[C@@]4(OC(=O)C(C4=O)=C([C@]5([C@H](C(=CC2)C)C=C[C@@H]6[C@@H](O[C@@H]7O[C@@H]([C@@H](OC(=O)C)[C@H](C7)O[C@H]8O[C@@H]([C@@H](O)CC8)C)C)[C@H](C)C[C@@H]([C@H]56)C)C)O)CC(C=O)=C[C@@H]3O)C)C)C', "
               "'Contains 1 aldehyde group'), "
               "('C=1(C(=C(C(=C(C1Cl)C)C=O)O)C/C=C(/CC/C=C(/[C@H](C[C@H]2C(C)(C)O2)O)\\\\C)\\\\C)[O-]', "
               "'Contains 1 aldehyde group'), "
               "('O=C1C=CC(=CC(C(OC(=O)CC(C(C(C(CC1C)CC=O)OC2OC(C(OC3OC(C(O)C(C3)(O)C)C)C(C2O)N(C)C)C)C)O)CC)COC4OC(C(O)C(C4OC)OC)C)C.O=C(O)C(O)C(O)C(=O)O', "
               "'Contains 1 aldehyde group'), "
               "('O=C1[C@H]2C(=CC[C@H]3C([C@H](/C=C\\\\C=C(C)C)C)CC[C@@]3(C[C@@H]2[C@@](C1)(O)C)C)C=O', "
               "'Contains 1 aldehyde group'), ('O=C(C(NCC(O)=O)CCCCC)C=O', "
               "'Contains 1 aldehyde group'), "
               "('O[C@H]1[C@H]([C@H](O[C@H]([C@@H]1O)O[C@@H]2[C@H](O[C@@H](OC[C@@H]([C@@H](/C=C/CCCCCCCCCCCCC)O)NC(=O)*)[C@@H]([C@H]2O)O)CO)CO)O[C@H]3[C@@H]([C@H]([C@@H](O)[C@H](O3)C=O)O)NC(C)=O', "
               "'Contains 1 aldehyde group'), ('OCC(CC=O)C([O-])=O', 'Contains "
               "1 aldehyde group'), "
               "('O=C1NC=2C(C(=O)[C@H]3OC3(C)C)=CC=CC2[C@]14[C@](C=O)([C@H]5C[C@H]6C[C@H](OC(=O)C)C[C@@H](N6C[C@@]5(C4)NC)C)C', "
               "'Contains 1 aldehyde group'), "
               "('O1[C@](O[C@@H]2[C@@H](O)[C@@H](O[C@@H]([C@@H]2O)CO)O[C@@H]([C@H](O[C@@H]3O[C@H]([C@@H](O)[C@@H](O)[C@@H]3O)C)[C@H](O)CO)[C@@H](NC(=O)C)C=O)(C[C@H](O)[C@@H](NC(=O)C)[C@@]1([C@H](O)[C@H](O)CO)[H])C(O)=O', "
               "'Contains 1 aldehyde group'), "
               "('O=CC1=C(O)C(=CC(=C1C#CC(=C)C)O)CC=C(C)C', 'Contains 1 "
               "aldehyde group'), ('O=CC1=C(CC[C@@H]2[C@@]1(CCCC2(C)C)C)CO', "
               "'Contains 1 aldehyde group'), ('[H]C(CS([O-])=O)=O', 'Contains "
               "1 aldehyde group'), "
               "('O=CC1=C(C=C2O[C@](CC[C@@H](O)C(O)(C)C)(C)[C@H]3[C@H]4C2=C1O[C@H]4[C@@](O)(CC3)C)C=O', "
               "'Contains 2 aldehyde groups')]\n"
               'False negatives: '
               "[('C(/C=[N+]\\\\1/C(CCC1)C([O-])=O)=C\\\\C=2C=C(C(=C(C2)OC)O)OC', "
               "'No aldehyde groups found'), "
               "('[C@H](C(NC)=O)(NC([C@H](CCCC=1C=CC=CC1)[C@@H](N(O)C=O)C)=O)C(C)(C)C', "
               "'No aldehyde groups found'), "
               "('C(=C/[NH+]1C(CCC1)C([O-])=O)\\\\C=C2C=C(C(C(=C2)OC)=O)OC', "
               "'No aldehyde groups found')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 100,
    'num_false_positives': 100,
    'num_true_negatives': 61,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.5,
    'recall': 0.9803921568627451,
    'f1': 0.662251655629139,
    'accuracy': 0.6121673003802282}


"""
Classifies: CHEBI:33560 p-block element atom
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_p_block_element_atom(smiles: str):
    """
    Determines if a molecule is a p-block element atom (main group element atom belonging to the p-block of the periodic table).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a p-block element atom, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    if mol.GetNumAtoms() != 1:
        return False, "Not a single atom"

    atom = mol.GetAtomWithIdx(0)
    atomic_number = atom.GetAtomicNum()
    symbol = atom.GetSymbol()
    mass_number = atom.GetMass()

    # Determine the group and period of the element
    period = AllChem.Atom.Atom(symbol).GetPeriod()
    group = AllChem.Atom.Atom(symbol).GetGroup()

    # Check if the element belongs to the p-block
    if 13 <= group <= 18 or group == 1:
        if mass_number == 0:
            return True, f"{symbol} is a p-block element atom"
        else:
            return True, f"{symbol}-{mass_number} is a p-block element atom"
    else:
        if mass_number == 0:
            return False, f"{symbol} is not a p-block element atom"
        else:
            return False, f"{symbol}-{mass_number} is not a p-block element atom"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33560',
                          'name': 'p-block element atom',
                          'definition': 'Any main group element atom belonging '
                                        'to the p-block of the periodic table.',
                          'parents': ['CHEBI:33318']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "type object 'Atom' has no attribute 'Atom'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:23798 dimethoxyflavone
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_dimethoxyflavone(smiles: str):
    """
    Determines if a molecule is a dimethoxyflavone (flavone with exactly 2 methoxy substituents).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a dimethoxyflavone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for flavone core structure (C15H10O2)
    matches = mol.GetSubstructMatches(Chem.MolFromSmarts('O=C1CC(=O)c2ccccc2O1'))
    if not matches:
        return False, "No flavone core structure found"
        
    # Count methoxy groups (-OCH3)
    methoxy_pattern = Chem.MolFromSmarts('OC')
    methoxy_matches = mol.GetSubstructMatches(methoxy_pattern)
    
    # Verify each methoxy match is actually -OCH3 (not part of larger group)
    true_methoxy_count = 0
    methoxy_positions = []
    
    for match in methoxy_matches:
        o_atom = mol.GetAtomWithIdx(match[0])
        c_atom = mol.GetAtomWithIdx(match[1])
        
        # Check if carbon has exactly 3 hydrogens
        if c_atom.GetTotalNumHs() == 3:
            # Get position number on flavone core
            for neighbor in o_atom.GetNeighbors():
                if neighbor.GetIdx() != match[1]:  # not the methyl carbon
                    methoxy_positions.append(neighbor.GetIdx())
                    true_methoxy_count += 1
                    
    if true_methoxy_count != 2:
        return False, f"Found {true_methoxy_count} methoxy groups, need exactly 2"
        
    return True, f"Dimethoxyflavone with methoxy groups at positions {methoxy_positions}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23798',
                          'name': 'dimethoxyflavone',
                          'definition': 'Any methoxyflavone with two methoxy '
                                        'substituents.',
                          'parents': ['CHEBI:25241']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183891,
    'num_false_negatives': 4,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999782484569999}


"""
Classifies: CHEBI:33653 aliphatic compound
"""
from rdkit import Chem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_aliphatic_compound(smiles: str):
    """
    Determines if a molecule is an aliphatic compound, defined as any acyclic or cyclic, saturated or unsaturated
    carbon compound, excluding aromatic compounds.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an aliphatic compound, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains any aromatic rings
    if mol.GetAromaticRingInfo().AromaticRings:
        return False, "Molecule contains aromatic rings, which are not aliphatic"

    # Check if the molecule contains any carbon atoms
    has_carbon = any(atom.GetSymbol() == 'C' for atom in mol.GetAtoms())
    if not has_carbon:
        return False, "Molecule does not contain any carbon atoms"

    # If the molecule passes the above checks, it is considered an aliphatic compound
    return True, "Molecule is an aliphatic compound"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33653',
                          'name': 'aliphatic compound',
                          'definition': 'Any acyclic or cyclic, saturated or '
                                        'unsaturated carbon compound, '
                                        'excluding aromatic compounds.',
                          'parents': ['CHEBI:50860']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "'Mol' object has no attribute 'GetAromaticRingInfo'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:139590 primary alpha-hydroxy ketone
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_primary_alpha_hydroxy_ketone(smiles: str):
    """
    Determines if a molecule contains a primary alpha-hydroxy ketone group.
    This is defined as a carbonyl group and hydroxy group linked by a CH2 (methylene) group.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains primary alpha-hydroxy ketone group, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for primary alpha-hydroxy ketone:
    # [OH]-[CH2]-C(=O)-[#6] 
    # This matches:
    # - OH group
    # - Connected to CH2 (methylene)
    # - Connected to C=O (carbonyl)
    # - Carbonyl connected to any carbon
    pattern = Chem.MolFromSmarts('[OH]-[CH2]-C(=O)-[#6]')
    
    if mol.HasSubstructMatch(pattern):
        matches = mol.GetSubstructMatches(pattern)
        # Get atoms involved in match
        match_atoms = matches[0]
        oh_atom = mol.GetAtomWithIdx(match_atoms[0])
        ch2_atom = mol.GetAtomWithIdx(match_atoms[1]) 
        co_atom = mol.GetAtomWithIdx(match_atoms[2])
        
        # Verify CH2 group has exactly 2 hydrogens
        if ch2_atom.GetTotalNumHs() != 2:
            return False, "Methylene carbon does not have exactly 2 hydrogens"
            
        # Verify carbonyl oxygen
        for atom in co_atom.GetNeighbors():
            if atom.GetSymbol() == 'O' and atom.GetTotalNumHs() == 0:
                return True, "Contains primary alpha-hydroxy ketone group"
                
        return False, "Carbonyl group not found"
    
    return False, "No primary alpha-hydroxy ketone pattern found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:139590',
                          'name': 'primary alpha-hydroxy ketone',
                          'definition': 'An alpha-hydroxy ketone in which the '
                                        'carbonyl group and the hydroxy group '
                                        'are linked by a -CH2 (methylene) '
                                        'group.',
                          'parents': ['CHEBI:139588', 'CHEBI:15734']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 8,
    'num_false_positives': 100,
    'num_true_negatives': 82007,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.07407407407407407,
    'recall': 1.0,
    'f1': 0.13793103448275862,
    'accuracy': 0.9987821957011508}


"""
Classifies: CHEBI:33250 atom
"""
from rdkit import Chem

def is_atom(smiles: str):
    """
    Determines if a molecule is an atom.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an atom, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule has only one atom
    if mol.GetNumAtoms() != 1:
        return False, "Not an atom (more than one atom present)"

    # Check if the SMILES string matches the atom symbol and isotope
    atom = mol.GetAtomWithIdx(0)
    atom_symbol = atom.GetSymbol()
    atom_isotope = atom.GetIsotope()

    smiles_atom = smiles.strip('[]')
    if smiles_atom.isdigit():
        smiles_isotope = int(smiles_atom)
        smiles_symbol = ''
    else:
        try:
            smiles_isotope = int(''.join(char for char in smiles_atom if char.isdigit()))
        except ValueError:
            smiles_isotope = 0
        smiles_symbol = ''.join(char for char in smiles_atom if not char.isdigit())

    if smiles_symbol.upper() != atom_symbol.upper():
        return False, "Not an atom (symbol mismatch)"

    if smiles_isotope != 0 and smiles_isotope != atom_isotope:
        return False, "Not an atom (isotope mismatch)"

    return True, "Molecule is an atom"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33250',
                          'name': 'atom',
                          'definition': 'A chemical entity constituting the '
                                        'smallest component of an element '
                                        'having the chemical properties of the '
                                        'element.',
                          'parents': ['CHEBI:24431']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: invalid literal for int() with base 10: ''",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 23,
    'num_false_positives': 12,
    'num_true_negatives': 183686,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.6571428571428571,
    'recall': 1.0,
    'f1': 0.7931034482758621,
    'accuracy': 0.9999346835691075}


"""
Classifies: CHEBI:134396 secondary allylic alcohol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_secondary_allylic_alcohol(smiles: str):
    """
    Determines if a molecule is a secondary allylic alcohol.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a secondary allylic alcohol, False otherwise
        str: Reason for classification
    """
    # Create RDKit mol object
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Add explicit hydrogens
    mol = Chem.AddHs(mol)
    
    # SMARTS pattern for secondary allylic alcohol:
    # [C]=[C]-[C@H]([*])([OH1])
    # This pattern looks for:
    # - A carbon-carbon double bond
    # - Connected to a carbon
    # - That carbon has exactly one hydrogen
    # - That carbon has a hydroxyl group
    # - That carbon has one other non-H substituent
    pattern = Chem.MolFromSmarts('[C]=[C][C@H]([*])[OH1]')
    
    if mol.HasSubstructMatch(pattern):
        matches = mol.GetSubstructMatches(pattern)
        
        for match in matches:
            central_c = mol.GetAtomWithIdx(match[2])  # The carbon attached to OH
            
            # Count the number of heavy atoms (non-H) attached
            heavy_neighbors = [n for n in central_c.GetNeighbors() if n.GetSymbol() != 'H']
            
            # For a secondary carbon, should have exactly 2 heavy atom neighbors (not counting OH)
            if len(heavy_neighbors) == 3:  # 3 because we're counting the OH group
                # Verify one neighbor is part of C=C double bond
                double_bond_found = False
                for neighbor in heavy_neighbors:
                    if neighbor.GetSymbol() == 'C':
                        for bond in neighbor.GetBonds():
                            if bond.GetBondType() == Chem.rdchem.BondType.DOUBLE:
                                other_atom = bond.GetOtherAtom(neighbor)
                                if other_atom.GetSymbol() == 'C':
                                    double_bond_found = True
                                    break
                
                if double_bond_found:
                    return True, "Secondary allylic alcohol found"
    
    return False, "No secondary allylic alcohol pattern found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:134396',
                          'name': 'secondary allylic alcohol',
                          'definition': 'An allylic alcohol in which the '
                                        'carbon atom that links the double '
                                        'bond to the hydroxy group is also '
                                        'attached to one other carbon and one '
                                        'hydrogen.',
                          'parents': ['CHEBI:134361', 'CHEBI:35681']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.1746031746031746 is too low.\n'
               'True positives: '
               "[('C=1(C(CC[C@@H](C1C)O)(C)C)/C=C/C(=C/C=C/C(=C/C(O)=O)/C)/C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C(\\\\CCO)=C\\\\C/C=C\\\\C/C=C\\\\C\\\\C=C/C=C/C(CCCC(=O)O)O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C=1(C(C[C@H](O)C1C)=O)C/C=C\\\\CC', 'Secondary carbon with "
               "OH group adjacent to C=C double bond'), "
               "('C(C(O)=O)C/C=C\\\\C/C=C\\\\C/C=C\\\\C=C\\\\[C@@H](C/C=C\\\\C/C=C\\\\CCO)O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C(C(O)=O)C/C=C\\\\C/C=C\\\\C/C=C\\\\CC(/C=C/C=C\\\\C/C=C\\\\CC)O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C(CCC[C@@H](/C=C/C=C\\\\C/C=C\\\\C=C\\\\[C@H](CCCCC)OO)O)(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('CCC(C)C1OC(=O)\\\\C=C\\\\C=C\\\\C=C\\\\C=C\\\\C=C\\\\CC(O)CC(O)CC(O)CC(O)CC(O)CC(O)CC(O)CC(O)\\\\C=C\\\\C1C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\[C@H](CCCC)O)O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C(C(O)=O)C/C=C\\\\C/C=C\\\\C/C=C\\\\C=C\\\\[C@H](C/C=C\\\\C/C=C\\\\CCO)O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C1[C@H]2[C@@H]([C@H](O[C@@H]1C2)/C=C/[C@H](CCC(CC)O)O)C/C=C\\\\CCCC(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C(CC/C=C\\\\C[C@@H]([C@@H](/C=C/C=C/C=C\\\\C=C\\\\[C@@H](C/C=C\\\\CC)O)O)O)(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double "
               "bond')]\n"
               'False positives: '
               "[('O=C1C(=C(/C=C/C(=C/C=C/C(=C/C=C/C=C(/C=C/C=C(/C=C/C=C(/C=C/[C@H](O)C(O[C@@H]2O[C@@H]([C@@H](O)[C@@H]([C@H]2O)O)CO)(C)C)\\\\C)\\\\C)\\\\C)/C)/C)C(C)(C)CC1)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1OC(C(C=CC=C(C(O)CC(O)CC(O)CC(OC(=O)CC(=O)O)CC2OC(CC(C(CCC(C(C(CC(C(C=CC=C1C)C)O)O)C)O)C)O)(O)C(O)C(C2)O)C)C)C(CCC/C=C/CCCN=C(N)N)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('CC\\\\C=C/C[C@H](O)\\\\C=C\\\\[C@@H]1[C@@H](C\\\\C=C/CCCC(O)=O)[C@@H](O)CC1=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('[H][C@@]12C[C@H](OC(C)=O)[C@@]3([H])[C@]45CCCC(C)(C)[C@@]4([H])[C@H](O)[C@](O)(O[C@@H]5OC)[C@@]3([C@H](O)C1=C)[C@@H]2O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@@H]1[C@]2([C@]([C@]3([C@@]([C@@]4([C@](C[C@H]3O)(C[C@H](O)CC4)[H])C)(C1)[H])[H])(CC[C@@]2([C@@H](CCC(O)C(C)=C)C)[H])[H])C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1O[C@@]2([C@@H](O)C[C@@H]3C([C@@H]2[C@H]1[C@H](O)/C(=C/CCC)/C)=CO[C@@]4([C@]56OC5(CC=C[C@@H]6O)C)[C@@H]3O4)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1[C@@H]([C@@H](C/C=C\\\\CCCC(O[C@H](COC(=O)CCCCCCC)CO)=O)C=C1)/C=C/[C@@H](O)CCCCC', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('ClC1=C(OC)C=C2CC(=CC=C[C@@H](OC)C(=O)C[C@@H]([C@H](C=C([C@H](CC(NC1=C2)=O)O)C)C)O)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C1[C@H]2[C@@H]([C@H]([C@@H]1OO2)/C=C/[C@H](CCCCC)O)CCCCCCC([O-])=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1O[C@H]2[C@@H](C)C([C@@]1([C@H](NC(=O)C(O)C)C=C(C=C[C@@H](O)CC=C(C=C[C@H](C2)OC(=O)C)C)C)C)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1C2=C([C@@]3(C(=O)C[C@@H]([C@]3(C1)C)[C@@H](CCC(=O)OCCCC)C)C)[C@@H](O)C[C@@H]4[C@@]2(CCC(C4(C)C)=O)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('CCCCC[C@H](O)\\\\C=C\\\\[C@H]1[C@H](O)CC(=O)[C@@H]1C\\\\C=C/CC(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@H]1[C@@H]([C@@H](CCCCCCC(O)=O)C(=O)C1)/C=C/[C@H](O)CCCCC', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O1C=2C(CC(O)C(C)=C)=C(OC)C=CC2C=CC1=O', 'Secondary carbon "
               "with OH group adjacent to C=C double bond'), "
               "('O[C@H]1[C@@H]([C@@H](CCCCC(OC)CC(O)=O)C(=O)C1)/C=C/[C@@H](O)C[C@H](CCCC)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@H](CCCCCCCC(OC)=O)/C=C/C=C/CCCCC', 'Secondary carbon "
               "with OH group adjacent to C=C double bond'), "
               "('O=C1O[C@@H](CC=CC(=O)CC=C[C@@H](C1)O)C', 'Secondary carbon "
               "with OH group adjacent to C=C double bond'), "
               "('CC(C)=CC(O)CC(=O)SCCNC(=O)CCNC(=O)C(O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1O[C@@H](CC[C@@H](O)C=CC(=O)O[C@H](C[C@@H](CC[C@H](C=C1)O)O)C)C[C@@H](O)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('OC(CC(C1CCC(=CC1)C)=C)C=C(C)C', 'Secondary carbon with OH "
               "group adjacent to C=C double bond'), "
               "('O1C(C1CCCC(O)=O)C[C@H]2[C@@H]([C@@H](O)CC2=O)/C=C/[C@@H](O)CCCCC', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O1[C@@]([C@@H](O)CC[C@H]([C@@]2([C@@]3([C@@](CC2)(/C(/CCC3)=C/C=C\\\\4/C[C@@H](O)C[C@H](O)C4=C)[H])C)[H])C)(C1)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1C2=C(C(=O)C[C@@H]3[C@@]2(CC[C@@H](C3(C)C)O)C)[C@@]4([C@@H](O)C=C([C@]4(C1)C)[C@@H](CC(=O)CC(C(=O)O)C)C)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('CC(=C)[C@@H](O)CC[C@](C)(O)[C@H]1CC[C@]2(C)[C@@H]1CC[C@@H]1[C@@]3(C)CCC(=O)C(C)(C)[C@@H]3CC[C@@]21C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@H]1CC[C@@]2([C@@]3([C@]([C@]4([C@@]([C@](CC4)([C@@H](CCCC(C)C)C)[H])(CC3)C)[H])(C=CC2=C1)[H])[H])C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1OC(C=CC(O)CC(O)CCCC(O)C(C(O)CCC(C(O)CC(O)CC(O)CCCC(C(C=C(C(CC(CCCC(C(C(CC=CC=C(C(C(C2OC(C1)(O)C(C(O[C@@H]3O[C@@H]([C@@H](O)[C@@H]([C@H]3O)N(C)C)C)C2)C)C)O)C)O)C)O)O)O)C)CCCCC(C)C)O)C)C)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O1C(C2C(C=3C1=CC(=CC3O)CCCCC)C=C(C(O)C2)C)(C)C', 'Secondary "
               "carbon with OH group adjacent to C=C double bond'), "
               "('CC\\\\C=C/C\\\\C=C/C=C/C(O)CCCCCCCC([O-])=O', 'Secondary "
               "carbon with OH group adjacent to C=C double bond'), "
               "('O[C@@H]1C=2[C@]3([C@@]([C@](CC3=O)([C@@H](CC(=O)C[C@@H](C)C(O)=O)C)[H])(CC(=O)C2[C@@]4([C@](C([C@@H](O)CC4)(C)C)(C1)[H])C)C)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C=C/C(O)CCC(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C1[C@H]2[C@@H]([C@H](O[C@@H]1C2)/C=C/[C@H](CCCC(C)O)O)C/C=C\\\\CCCC(=O)[O-]', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCC(C)C)COC(=O)CCC/C=C\\\\C[C@@H]1[C@H]([C@H](O)CC1=O)/C=C/[C@@H](O)CCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1O[C@]([C@@]2(CC=C(C)[C@H](C2)O)C)(C)CC1', 'Secondary "
               "carbon with OH group adjacent to C=C double bond'), "
               "('CC(C)C1=C(C(=C(C(=N1)C(C)C)C=C[C@H](C[C@H](CC(=O)O)O)O)C2=CC=C(C=C2)F)COC', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('[C@@H]1(C[C@@H](C/C(/C1=C)=C/C=C\\\\2/[C@]3([C@](CCC2)([C@](CC3)([C@@H](/C=C/[C@@H](O)C4CC4)C)[H])C)[H])O)O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('OCCCCC\\\\C=C/C[C@@H](O)C(O)\\\\C=C\\\\C(O)C\\\\C=C/CCCC(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P(OC[C@H](OC(=O)CCC/C=C\\\\C[C@@H]1[C@H](C(=O)C=C1)/C=C/[C@@H](O)CCCCC)COC(=O)CCCCCCCCCCCCCCCCCCCC)(O)(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('ClC1=C(O)C=C(O)C2=C1CC(=O)C[C@@H](O)C=CCCC=CC[C@H](OC2=O)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('CCCCC[C@@H](C=C[C@H]1[C@@H]([C@H](CC1=O)O)CC=CCCCC(=O)O)O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@](\\\\C=C\\\\[C@H]([C@@]1([C@@]2([C@@](CC1)(/C(/CCC2)=C/C=C\\\\3/C[C@@H](O)C[C@H](O)C3=C)[H])C)[H])C)(C(O)(C)C)CO', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@@H]1CC=2[C@@]([C@@]3([C@]([C@]4([C@@]([C@](CC4)([C@@H](C[C@H](O)C(C(C)C)=C)C)[H])(CC3)C)[H])(CC2)[H])[H])(CC1)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1OC2C3(C4(C5C(C(O)(C)CC4)CC3(O)C(C2)O5)COC(=O)C6C7(C(C(=CCC=C1)OCC7)O)O6)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P(OC[C@H](NC(=O)C/C=C\\\\C[C@H]1[C@H]([C@H](O)C[C@@H]1O)/C=C/[C@H](O)CCCCC)[C@H](O)/C=C/CCCCCCCCCCCCCC)(OCC[N+](C)(C)C)([O-])=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P(OC[C@H](OC(=O)CCCCCC[C@@H]1[C@H]([C@H](O)CC1=O)/C=C/[C@@H](O)CCCCC)COC(=O)CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(OCC[N+](C)(C)C)([O-])=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P(OC[C@H](NC(=O)CCC(O)\\\\C=C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)[C@H](O)CCCCCCCCCCCCCCC)(OCC[N+](C)(C)C)([O-])=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P(OC[C@H](OC(=O)CCCCCC[C@@H]1[C@H]([C@H](O)C[C@@H]1O)/C=C/[C@@H](O)CCCCC)COC(=O)CCCCCCCCCCCCCCCCCCCCC(CC)C)(OC[C@@H](O)CO)(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@H]1[C@@H]([C@@H]([C@@H](O)C1)/C=C/[C@@H](O)CC)C/C=C\\\\C/C=C\\\\C/C=C\\\\CCC(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@@H](COc1cccc(c1)C(F)(F)F)\\\\C=C\\\\[C@H]1[C@H](O)C[C@H](O)[C@@H]1C\\\\C=C/CCCC(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1N[C@@H]([C@H](O)C)C([C@@]12C(=O)[C@]3([C@H]4[C@@H](C[C@H](C)CC4)C=C[C@H]3[C@@H]2/C=C/[C@H](O)C)C)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C(O[C@@H](CCC(=O)/C=C/C(=O)O)C)/C=C/[C@@H](O)CC[C@H](OC(=O)C)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1C2=C([C@@]3([C@@H](O)C[C@@H]([C@]3(C1)C)[C@@H](CC(O)C=C(C(=O)O)C)C)C)[C@@H](O)CC4[C@@]2(CC[C@@H](C4(C)C)O)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('OC(/C=C\\\\CCCCCCC)C#CC#CC(=O)C=C', 'Secondary carbon with "
               "OH group adjacent to C=C double bond'), "
               "('OC1C23C(C4(C(C2C(O)=O)C(CCC4)(C)C(O)=O)C)CCC(O)(C3)C1=C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1OC(C2=CC=CC=C2)C(C1NC(=O)C(NC(=O)C(C(OC)CC(O)/C(=C/CC3OC(/C=C/C=C/CC(OC)CC(=O)/C=C/C(=C/C=C/C4C(O)C(O)(C(C)OC4)C)/C)C(C)C(C3CC)O)/C)C)C(C)C)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1OC(C=CC(O)CC(O)CCCC(O)C(C(O)CCC(C(O)CC(O)CC(O)CCCC(C(C=C(C(CC(CCCC(C(C(CC=CC=C(C(C(C2OC(C1)(O)C(C(O[C@@H]3O[C@@H]([C@@H](O)[C@@H]([C@H]3O)N(C)C)C)C2)C)C)O)C)O)C)O)O)O)C)CCCC(CC)C)O)C)C)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C(C(CC/C=C/C=C\\\\[C@H](CCCC([O-])=O)O)=O)/C=C\\\\CCCCC', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O1[C@H](/C=C/C=C/[C@H](O)C)[C@H]2C[C@@H](O)[C@@H]([C@@H]([C@@H]2C1)O)O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1O[C@H](CCCC=CC=C[C@H](O)C[C@@H](O)CC=CC=C[C@H](CC=CC=C1)OC(=O)/C=C/C)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCCCCC(O)/C=C/C=C/C/C=C/CC)COC(=O)CCCCCCCCCCCCCCCCCCCCCCC)([O-])=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('[H][C@@]1(CC[C@]2(C)[C@]1([H])[C@H](O)C[C@]1([H])[C@@]3(C)CC[C@H](O)C(C)(C)[C@]3([H])[C@@H](O)C[C@@]21C)[C@@](C)(O)C(O)C[C@H](O)C(C)=C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1C(=C(C=CC(O)(C)C)[C@@H](O)CC1)CC=C(C)C', 'Secondary "
               "carbon with OH group adjacent to C=C double bond'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCC(C)C)COC(=O)CCCC(O)\\\\C=C\\\\C=C\\\\C\\\\C=C\\\\CCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1O[C@H]2C(C)(C)C([C@@]1([C@@]2(/C=C/C=C/C=C(\\\\C=C(\\\\[C@@H](O)/C(=C/C)/C)/C)/C)C)C)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('C(=C/[C@@H](C#N)O)\\\\C', 'Secondary carbon with OH group "
               "adjacent to C=C double bond'), "
               "('FC(F)(F)C(O)(C#CC[C@@H]([C@@]1([C@@]2([C@@](CC1)(/C(/CCC2)=C/C=C\\\\3/C[C@@H](O)C[C@H](O)C3=C)[H])C)[H])CCCC(O)(C)C)C(F)(F)F', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@@H]1[C@H]([C@H]([C@H](O)C1)/C=C/[C@H](O)CCCCC)C/C=C\\\\CC(O[C@H](COC(=O)CCCCCCCCCCCCCCCCCCCC)CO)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O([C@@H]1CC=2[C@@]([C@@H]3[C@H]([C@H]4[C@@]([C@H](CC4)[C@@H](CCCC(C)C)C)(CC3)C)CC2)(CC1)C)C(=O)CCCCC(=O)C[C@@H]5[C@H]([C@H](O)C[C@@H]5O)/C=C/[C@@H](O)CCCCC', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('CC1OC(OC2C\\\\C=C(CCC=C)/CC\\\\C=C\\\\C(O)C(C)C(O)C(C)\\\\C=C\\\\C=C\\\\C=C\\\\C=C\\\\C=C\\\\CCC(=O)OC(CC(O)CC(O)CC(O)C\\\\C=C\\\\C(O)CC(O)CC(O)CC(=O)C2C)\\\\C=C\\\\CC(O)\\\\C=C\\\\CC(O)CCCN)C(O)C(O)C1O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1O[C@@H](C[C@H](O)C[C@@H](O)CCCCCCCCCCCC[C@@H](C[C@@H](C=C1)O)O)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1O[C@H]([C@H](C=C[C@H](O)C[C@H](O)C[C@@H](O)C[C@H](O)C[C@@H](O)C[C@H](C[C@H]([C@H]([C@H]2O[C@H](C=CC=CC=CC=C1)[C@H](O)C2)C)O)O)C)C(CC)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('S1S[C@@]23N4[C@@H]5[C@@H](OC(=O)C=6C=CC(=C(OC=7C=C(C[C@]1(N(C)C2=O)C4=O)C=CC7OC)C6)OC)C=COC=C5[C@H]3O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P1(O[C@H]2[C@H](O)[C@@H](O)[C@H](O)[C@H]([C@@H](O)CC(O[C@@H]([C@@H](O)[C@H]2OP(O)(O)=O)C=C[C@@H](O)CCCCC)O)CC=CCCCC(O[C@@H](CO1)COC(=O)CCCCCCC/C=C\\\\CCCCCC)=O)(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1C2=C([C@@]3(O)CCCC(C3=C1)(C)C)CC[C@@]([C@H]2O)(C=C)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@H](CCCC(O)=O)/C=C\\\\C=C\\\\C=C\\\\[C@@H](O)C/C=C\\\\CCC(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@@H]1C\\\\C(=C\\\\C=C/2\\\\[C@]3([C@@]([C@](CC3)([C@H](C)/C=C/C=C/C=C/CO)[H])(CCC2)C)[H])\\\\C([C@@H](O)C1)=C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P(OC[C@H](OC(=O)CCC(O)/C=C/C=O)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)(O)(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1/C(=C/[C@@H](O)C)/C[C@H](O)[C@@]23[C@@]1(OC(C)(C)[C@@H]2C3)O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@@H]1C/C(=C\\\\C=C/2\\\\[C@H]3[C@@]([C@H](CC3)[C@H](CC)C)(CCC2)C)/C[C@@H](O)C1=C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1C2=C(O[C@@H](C2)/C=C/CCCCCC)[C@H](O)CC1', 'Secondary "
               "carbon with OH group adjacent to C=C double bond'), "
               "('[C@]1([C@H](C(=C(C(C1)=O)O)[O-])O)(CO)O', 'Secondary carbon "
               "with OH group adjacent to C=C double bond'), "
               "('OC1C(C(C(=O)C1)C/C=C/CCC(OC(C[N+](C)(C)C)CC([O-])=O)=O)/C=C/C(O)C/C=C\\\\C/C=C/CC', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P1(O[C@H]2[C@H](O)[C@@H](O)[C@H](O)[C@H]([C@@H](O)CC(O[C@@H]([C@@H](O)[C@H]2OP(O)(O)=O)/C=C/[C@@H](O)CCCCC)O)CC=CCCCC(OC[C@@H](OC(=O)CCCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCCC)CO1)=O)(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@@]12[C@@](C=3C([C@H]4[C@]([C@@H]([C@@H](/C=C/[C@@H](C(C)C)C)C)CC4)(C)CC3)=C[C@H]1O)(CC[C@@H](C2)O)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O1C(C(NC2=CC=C(C=C2)C=O)CC1=O)C=CC(O)C', 'Secondary carbon "
               "with OH group adjacent to C=C double bond'), "
               "('C[C@H]1[C@H]2[C@H](Cc3c[nH]c4ccccc34)NC(=O)[C@@]22[C@H](C=C1C)\\\\C=C\\\\C[C@H](C)\\\\C=C(C)\\\\[C@@H](O)C(=O)\\\\C=C\\\\C2=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('COc1cc2c(CN3CC[C@@]22C=C[C@@H](O)C[C@H]32)cc1O', 'Secondary "
               "carbon with OH group adjacent to C=C double bond'), "
               "('O=C1[C@@]23C(=O)N[C@H]([C@@H]2C(C)=C([C@H]([C@@H]3C=CC[C@H](C)C=C([C@H](C(CC1)=O)O)C)OC)C)CC=4C5=C(C=CC=C5)NC4', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O1[C@@H](OC[C@]2([C@@H](O)[C@H](O)C[C@]3([C@H]2C[C@@H](O)C4=C3CC[C@@](C4)(C=C)C)C)C)[C@@H](O)[C@H](O)[C@@H]([C@H]1CO)O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C([C@H](O)/C(/C=O)=C\\\\C=C\\\\C)C', 'Secondary carbon "
               "with OH group adjacent to C=C double bond'), "
               "('O=C1C(=C(C)C)C[C@@]2([C@H](CCC(C2=C1)O)C)C', 'Secondary "
               "carbon with OH group adjacent to C=C double bond'), "
               "('O=C1C2=C(C(O)=C(C)C=C2C(=O)C=3C1=C(O)C=C(O)C3)C4=C5C(=O)C6=C([C@@H](OC)[C@@H](O)[C@]([C@H]6O)(O)C)C(C5=C(O)C=C4OC)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P(OC[C@H](NC(=O)CCC/C=C\\\\C[C@@H]1[C@H]([C@H](O)CC1=O)/C=C/[C@@H](O)CCCCC)[C@H](O)/C=C/CCCCCCCCCCCCC)(OCC[N+](C)(C)C)([O-])=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('OC(C(O)CC(O)/C(=C/CC/C(=C\\\\CO)/C)/C)(C)C', 'Secondary "
               "carbon with OH group adjacent to C=C double bond'), "
               "('CCCCCC(O)C=CC(O)=O', 'Secondary carbon with OH group "
               "adjacent to C=C double bond'), "
               "('O=C1O[C@@H](CC=C[C@H](CCC1)O)C', 'Secondary carbon with OH "
               "group adjacent to C=C double bond'), "
               "('O=C1NC=C(C2=CC=CC=C2)C3=C1[C@](/C=C(/C(O)C)\\\\C)([C@H](O)O3)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('P(OC[C@H](OC(=O)CCC[C@@H](O)[C@H](O)/C=C\\\\C=C/C=C/C=C/[C@@H](O)C/C=C\\\\CC)COC(=O)CCCCCCCCCCCCCCCCCCCCC(C)C)(OC[C@@H](O)CO)(O)=O', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O[C@H](\\\\C=C\\\\CCCCCCCCC\\\\C=C/CCCCCCCCC\\\\C=C\\\\[C@@H](O)C#C)C#C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1C2=C([C@@]3(CC[C@@H]([C@]3(C1)C)[C@@H](CC/C=C(/C(=O)O)\\\\C)C)C)[C@@H](O)C[C@@H]4[C@@]2(CCC([C@]4(CO)C)=O)C', "
               "'Secondary carbon with OH group adjacent to C=C double bond'), "
               "('O=C1[C@@]2(O[C@@H]2[C@H](C(=O)OC)O[C@H]1O)[C@H](O)/C=C(/[C@@H]3OC(C)(C)[C@@H]4[C@H]3C=C(CC4)C)\\\\C', "
               "'Secondary carbon with OH group adjacent to C=C double "
               "bond')]\n"
               'False negatives: '
               "[('O=C(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C(C1C(CCCCC)O1)O)O', 'No "
               "secondary allylic alcohol pattern found'), "
               "('CCCCCC(O)C(O)C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCC(O)=O', 'No "
               "secondary allylic alcohol pattern found'), "
               "('C1=CC=C(C(=C1/C=C/C=C/[C@H]([C@H](C)O)O)C([H])=O)O', 'No "
               "secondary allylic alcohol pattern found'), "
               "('CC1C(O)C=C2C1(C)C=CCC2(C)C', 'No secondary allylic alcohol "
               "pattern found')]",
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 14,
    'num_false_positives': 100,
    'num_true_negatives': 1962,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.12280701754385964,
    'recall': 0.9333333333333333,
    'f1': 0.21705426356589147,
    'accuracy': 0.9513721714010592}


"""
Classifies: CHEBI:27300 vitamin D
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_vitamin_D(smiles: str):
    """
    Determines if a molecule is a vitamin D compound.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a vitamin D compound, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of hydroxyl groups
    num_hydroxyl_groups = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O' and atom.GetIsAromatic() == False)
    if num_hydroxyl_groups < 1:
        return False, "No hydroxyl groups found"

    # Check for the presence of a seco-steroid skeleton
    seco_steroid_skeleton = False
    rings = mol.GetRingInfo().AtomRings()
    for ring in rings:
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetSymbol() == 'C' for atom in atoms):
                seco_steroid_skeleton = True
                break

    if not seco_steroid_skeleton:
        return False, "No seco-steroid skeleton found"

    # Check for the presence of a conjugated triene system
    conjugated_triene = False
    for bond in mol.GetBonds():
        begin_atom = bond.GetBeginAtom()
        end_atom = bond.GetEndAtom()
        begin_idx = begin_atom.GetIdx()
        end_idx = end_atom.GetIdx()
        bond_type = bond.GetBondType()
        if bond_type == Chem.BondType.DOUBLE:
            neighbors_begin = [neighbor.GetIdx() for neighbor in begin_atom.GetNeighbors()]
            neighbors_end = [neighbor.GetIdx() for neighbor in end_atom.GetNeighbors()]
            if any(mol.GetBondBetweenAtoms(begin_idx, neighbor).GetBondType() == Chem.BondType.DOUBLE for neighbor in neighbors_begin) and \
               any(mol.GetBondBetweenAtoms(end_idx, neighbor).GetBondType() == Chem.BondType.DOUBLE for neighbor in neighbors_end):
                conjugated_triene = True
                break

    if not conjugated_triene:
        return False, "No conjugated triene system found"

    return True, "Molecule is a vitamin D compound"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27300',
                          'name': 'vitamin D',
                          'definition': 'Any member of a group of fat-soluble '
                                        'hydroxy seco-steroids that exhibit '
                                        'biological activity against vitamin D '
                                        'deficiency. Vitamin D  can be '
                                        'obtained from sun exposure, food and '
                                        'supplements and is biologically '
                                        'inactive and converted into the '
                                        'biologically active calcitriol via '
                                        'double hydroxylation in the body.',
                          'parents': ['CHEBI:36853']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: cannot unpack non-iterable Bond object',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 25,
    'num_false_positives': 100,
    'num_true_negatives': 109,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.2,
    'recall': 1.0,
    'f1': 0.33333333333333337,
    'accuracy': 0.5726495726495726}


"""
Classifies: CHEBI:24803 indole-3-acetic acids
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_indole_3_acetic_acids(smiles: str):
    """
    Determines if a molecule is an indole-3-acetic acid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an indole-3-acetic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Pattern for indole core with acetic acid at position 3
    # Matches indole with -CH2COOH at position 3
    indole_pattern = Chem.MolFromSmarts('c1ccc2c(c1)[nH]cc2CC(=O)O')
    
    if not mol.HasSubstructMatch(indole_pattern):
        # Try alternate pattern with different bond representation
        indole_pattern2 = Chem.MolFromSmarts('[cH]1[cH][cH][c]2[c]1[nH][cH][c]2CC(=O)O')
        if not mol.HasSubstructMatch(indole_pattern2):
            return False, "Not an indole-3-acetic acid structure"

    # Check for carboxylic acid group
    carboxyl_pattern = Chem.MolFromSmarts('CC(=O)O')
    if not mol.HasSubstructMatch(carboxyl_pattern):
        return False, "Missing acetic acid group"

    # Verify position of acetic acid group
    matches = mol.GetSubstructMatches(indole_pattern)
    if not matches:
        matches = mol.GetSubstructMatches(indole_pattern2)

    # Check for aromaticity
    ring_info = mol.GetRingInfo()
    rings = ring_info.AtomRings()
    aromatic_rings = 0
    for ring in rings:
        if len(ring) in [5,6]:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetIsAromatic() for atom in atoms):
                aromatic_rings += 1
    
    if aromatic_rings < 2:
        return False, "Indole core not properly aromatic"

    # Check for substituents
    match_atoms = set(matches[0])
    substituents = []
    
    for atom in mol.GetAtoms():
        if atom.GetIdx() not in match_atoms and atom.GetSymbol() not in ['H', 'O']:
            substituents.append(atom.GetSymbol())
            
    if substituents:
        return True, f"Substituted indole-3-acetic acid with substituents: {', '.join(set(substituents))}"
    else:
        return True, "Unsubstituted indole-3-acetic acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24803',
                          'name': 'indole-3-acetic acids',
                          'definition': 'An  indol-3-yl carboxylic acid in '
                                        'which the carboxylic acid specified '
                                        'is acetic acid.',
                          'parents': ['CHEBI:24810']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               "False negatives: [('O=C(O)CC=1C2=C(C=C(CC=C(C)C)C=C2)NC1', "
               "'Not an indole-3-acetic acid structure'), "
               "('OC(=O)CC=1C=2C(NC1C)=CC=CC2', 'Not an indole-3-acetic acid "
               "structure'), ('O(C1=CC=2C(=C(NC2C=C1)C)CC(O)=O)C', 'Not an "
               "indole-3-acetic acid structure')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 18,
    'num_true_negatives': 183881,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.14285714285714285,
    'recall': 1.0,
    'f1': 0.25,
    'accuracy': 0.9999021217822536}


"""
Classifies: CHEBI:139477 mogroside
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_mogroside(smiles: str):
    """
    Determines if a molecule is a mogroside (a triterpenoid saponin in which the triterpenoid fragment can be any cucurbitane derivative).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a mogroside, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a cucurbitane core
    cucurbitane_core = '[C@@]12([C@](C3([C@@]([C@](CC3)(C1)C)(CC[C@@H](O)[C@H]2C)C)C)CC'
    cucurbitane_core_mol = Chem.MolFromSmarts(cucurbitane_core)
    if not mol.HasSubstructMatch(cucurbitane_core_mol):
        return False, "No cucurbitane core found"

    # Check for the presence of a triterpenoid saponin fragment
    saponin_fragment = '[C@H]([C@H](O)[C@@H](O)[C@H](O)O)[C@H](O)[C@H](O)CO'
    saponin_fragment_mol = Chem.MolFromSmarts(saponin_fragment)
    if not mol.HasSubstructMatch(saponin_fragment_mol):
        return False, "No triterpenoid saponin fragment found"

    return True, "Molecule is a mogroside (triterpenoid saponin with cucurbitane core)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:139477',
                          'name': 'mogroside',
                          'definition': 'A triterpenoid saponin in which the '
                                        'triterpenoid fragment can be any '
                                        'cucurbitane derivative.',
                          'parents': ['CHEBI:61778']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.HasSubstructMatch(Mol, NoneType)\n'
             'did not match C++ signature:\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle query, '
             'RDKit::SubstructMatchParameters params=True)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
             'RDKit::SubstructMatchParameters params)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle query, '
             'bool recursionPossible=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
             'bool recursionPossible=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:33447 phospho sugar
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_phospho_sugar(smiles: str):
    """
    Determines if a molecule is a phospho sugar, defined as any monosaccharide
    containing an alcoholic hydroxy group esterified with phosphoric acid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a phospho sugar, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains a phosphate group
    has_phosphate = any(atom.GetSymbol() == 'P' and
                        sum(bond.GetBondTypeAsDouble() for bond in atom.GetBonds()) == 5 for atom in mol.GetAtoms())
    if not has_phosphate:
        return False, "Molecule does not contain a phosphate group"

    # Check if the molecule is a monosaccharide
    ring_info = mol.GetRingInfo()
    ring_sizes = [len(ring) for ring in ring_info.AtomRings()]
    has_furanose_or_pyranose = (5 in ring_sizes) or (6 in ring_sizes)
    if not has_furanose_or_pyranose:
        return False, "Molecule is not a monosaccharide"

    # Check if the phosphate group is esterified to an alcoholic hydroxy group
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'P':
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'O' and neighbor.GetTotalNumHs() == 1:
                    return True, "Molecule is a phospho sugar"

    return False, "Phosphate group is not esterified to an alcoholic hydroxy group"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33447',
                          'name': 'phospho sugar',
                          'definition': 'Any monosaccharide containing an '
                                        'alcoholic hydroxy group esterified '
                                        'with phosphoric acid.',
                          'parents': ['CHEBI:26816', 'CHEBI:63367']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 42,
    'num_false_positives': 100,
    'num_true_negatives': 5077,
    'num_false_negatives': 18,
    'num_negatives': None,
    'precision': 0.29577464788732394,
    'recall': 0.7,
    'f1': 0.41584158415841577,
    'accuracy': 0.9774680160397174}


"""
Classifies: CHEBI:25996 phenylhydrazines
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_phenylhydrazines(smiles: str):
    """
    Determines if a molecule is a phenylhydrazine (hydrazine with phenyl substituent).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a phenylhydrazine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Define phenylhydrazine pattern - phenyl ring connected to N-N group
    phenylhydrazine_pattern = Chem.MolFromSmarts('c1ccccc1-[NX3;H2,H1,H0]-[NX3;H2,H1,H0]')
    
    # Find phenylhydrazine groups
    matches = mol.GetSubstructMatches(phenylhydrazine_pattern)
    
    if not matches:
        return False, "No phenylhydrazine group found"
        
    # Count number of phenylhydrazine groups
    num_matches = len(matches)
    
    # Get substituents on hydrazine nitrogens
    substituents = set()
    for match in matches:
        # The N atoms are at indices 6 and 7 in the pattern
        for n_idx in [match[6], match[7]]:
            n_atom = mol.GetAtomWithIdx(n_idx)
            for neighbor in n_atom.GetNeighbors():
                # Skip the N-N bond and phenyl connection
                if neighbor.GetIdx() not in [match[6], match[7]] + list(match[:6]):
                    substituents.add(neighbor.GetSymbol())
                    
    if substituents:
        return True, f"Found {num_matches} phenylhydrazine group(s) with substituents: {', '.join(substituents)}"
    else:
        return True, f"Found {num_matches} unsubstituted phenylhydrazine group(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25996',
                          'name': 'phenylhydrazines',
                          'definition': 'Any member of the class of  '
                                        'hydrazines carrying a phenyl '
                                        'substituent.',
                          'parents': ['CHEBI:24631']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: No module named 'rdkit.Chem.rdDecomposition'",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 176159,
    'num_false_negatives': 4,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 0.2,
    'f1': 0.018867924528301886,
    'accuracy': 0.9994099759451731}


"""
Classifies: CHEBI:17375 (2S)-2-hydroxy monocarboxylic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors


def is__2S__2_hydroxy_monocarboxylic_acid(smiles: str):
    """
    Determines if a molecule is a (2S)-2-hydroxy monocarboxylic acid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a (2S)-2-hydroxy monocarboxylic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for carboxylic acid group
    carboxylic_acid_smarts = "[$(C(=O)O)]"
    carboxylic_acid_atoms = mol.GetSubstructMatches(Chem.MolFromSmarts(carboxylic_acid_smarts))
    if not carboxylic_acid_atoms:
        return False, "No carboxylic acid group found"

    # Check for hydroxyl group
    hydroxyl_smarts = "[OX1H]"
    hydroxyl_atoms = mol.GetSubstructMatches(Chem.MolFromSmarts(hydroxyl_smarts))
    if not hydroxyl_atoms:
        return False, "No hydroxyl group found"

    # Check if the hydroxyl group is attached to carbon 2
    carboxylic_acid_atom = mol.GetAtomWithIdx(carboxylic_acid_atoms[0][0])
    carboxylic_acid_carbon = carboxylic_acid_atom.GetNeighbors()[0]
    hydroxyl_carbon = mol.GetAtomWithIdx(hydroxyl_atoms[0][0]).GetNeighbors()[0]
    if carboxylic_acid_carbon.GetIdx() != hydroxyl_carbon.GetIdx():
        return False, "Hydroxyl group not attached to carbon 2"

    # Check for (S)-configuration
    chiral_atom = hydroxyl_carbon
    if chiral_atom.GetChiralTag() != Chem.ChiralType.CHI_TETRAHEDRAL_CCW:
        return False, "Carbon 2 not (S)-configured"

    return True, "(2S)-2-hydroxy monocarboxylic acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17375',
                          'name': '(2S)-2-hydroxy monocarboxylic acid',
                          'definition': 'A 2-hydroxy monocarboxylic acid in '
                                        'which the carbon at position 2 has '
                                        '(S)-configuration.',
                          'parents': ['CHEBI:49302']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'tuple index out of range',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:33572 resorcinols
"""
from rdkit import Chem
from rdkit.Chem import rdMolDescriptors

def is_resorcinol(smiles: str):
    """
    Determines if a molecule is a resorcinol (benzenediol with hydroxy groups meta to each other).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a resorcinol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule has a single benzene ring
    ring_info = mol.GetRingInfo()
    aromatic_rings = [ring for ring in ring_info.AtomRings() if all(mol.GetAtomWithIdx(idx).GetIsAromatic() for idx in ring)]
    benzene_rings = [ring for ring in aromatic_rings if len(ring) == 6]

    if len(benzene_rings) != 1:
        return False, "Molecule does not have a single benzene ring"

    # Check if the benzene ring has two hydroxy substituents
    benzene_ring = benzene_rings[0]
    hydroxy_atoms = []
    for idx in benzene_ring:
        atom = mol.GetAtomWithIdx(idx)
        if atom.GetSymbol() == 'O' and atom.GetTotalNumHs() == 1:
            hydroxy_atoms.append(atom)

    if len(hydroxy_atoms) != 2:
        return False, "Benzene ring does not have exactly two hydroxy substituents"

    # Check if the hydroxy groups are meta to each other
    distance_matrix = Chem.GetDistanceMatrix(mol)
    hydroxy_idxs = [hydroxy_atom.GetIdx() for hydroxy_atom in hydroxy_atoms]
    distance = distance_matrix[hydroxy_idxs[0], hydroxy_idxs[1]]

    if distance != 3:
        return False, "Hydroxy groups are not meta to each other"

    return True, "Molecule is a resorcinol"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33572',
                          'name': 'resorcinols',
                          'definition': 'Any benzenediol in which the two '
                                        'hydroxy groups are meta to one '
                                        'another.',
                          'parents': ['CHEBI:33570']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "name 'is_resorcinols' is not defined",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:16158 steroid sulfate
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprint
from rdkit.Chem import rdMolDescriptors

def is_steroid_sulfate(smiles: str):
    """
    Determines if a molecule is a steroid sulfate.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a steroid sulfate, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of sulfate group (-OSO3)
    sulfate_pattern = Chem.MolFromSmarts('OS(=O)(=O)O')
    if not mol.HasSubstructMatch(sulfate_pattern):
        return False, "No sulfate group found"

    # Basic steroid scaffold patterns
    steroid_patterns = [
        # Generic steroid core (4 fused rings)
        Chem.MolFromSmarts('[#6]~1~[#6]~[#6]~[#6]~2~[#6]~[#6]~[#6]~3~[#6]~[#6]~[#6]~4~[#6]~[#6]~[#6]~[#6]~4~[#6]~[#6]~3~[#6]~[#6]~2~[#6]~1'),
        # Androstane skeleton
        Chem.MolFromSmarts('C12CCC3C(C1)CCC4C3(CCC4)C2'),
        # Estrane skeleton  
        Chem.MolFromSmarts('C12CCC3c4ccccc4CCC3C1CCC2'),
        # Pregnane skeleton
        Chem.MolFromSmarts('CC(C)CCCC(C)C1CCC2C1(CCC3C2CCC4=CC(=O)CCC34C)C')
    ]

    # Check if molecule contains any of the steroid patterns
    has_steroid = False
    for pattern in steroid_patterns:
        if pattern is not None and mol.HasSubstructMatch(pattern):
            has_steroid = True
            break
            
    if not has_steroid:
        return False, "No steroid scaffold found"

    # Additional checks to confirm steroid nature
    ring_info = mol.GetRingInfo()
    if len(ring_info.AtomRings()) < 4:
        return False, "Insufficient number of rings for steroid"

    # Count carbons
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetAtomicNum() == 6)
    if carbon_count < 17:  # Steroids typically have 17+ carbons
        return False, "Too few carbons for steroid structure"

    return True, "Contains steroid scaffold with sulfate group"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:16158',
                          'name': 'steroid sulfate',
                          'definition': 'A sulfuric ester obtained by the '
                                        'formal condensation of a hydroxy '
                                        'group of any steroid with sulfuric '
                                        'acid.',
                          'parents': [   'CHEBI:25704',
                                         'CHEBI:26819',
                                         'CHEBI:47880']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 18,
    'num_true_negatives': 183836,
    'num_false_negatives': 7,
    'num_negatives': None,
    'precision': 0.05263157894736842,
    'recall': 0.125,
    'f1': 0.07407407407407407,
    'accuracy': 0.9998640284561247}


"""
Classifies: CHEBI:18222 xylose
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_xylose(smiles: str):
    """
    Determines if a molecule is an xylose (an aldopentose).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an xylose, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule has 5 carbon atoms
    if mol.GetNumAtoms() != 16:  # 5 carbons, 5 oxygens, 6 hydrogens
        return False, "Molecule does not have the correct number of atoms for xylose"

    # Check if the molecule has an aldehyde group
    aldehyde_atoms = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetAtomicNum() == 8 and atom.GetTotalNumHs() == 0]
    if len(aldehyde_atoms) != 1:
        return False, "Molecule does not contain a single aldehyde group"

    # Check if the molecule has 4 hydroxyl groups
    hydroxyl_atoms = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetAtomicNum() == 8 and atom.GetTotalNumHs() == 1]
    if len(hydroxyl_atoms) != 4:
        return False, "Molecule does not contain 4 hydroxyl groups"

    # Check if the molecule is pentose (5-membered ring)
    ring_info = mol.GetRingInfo()
    rings = ring_info.AtomRings()
    if not any(len(ring) == 5 for ring in rings):
        return False, "Molecule does not contain a 5-membered ring"

    # Check if the molecule is an aldose (aldehyde carbon is part of the ring)
    aldehyde_atom = mol.GetAtomWithIdx(aldehyde_atoms[0])
    ring_atoms = rings[0]  # Assuming only one ring
    if aldehyde_atom.GetIdx() not in ring_atoms:
        return False, "Aldehyde carbon is not part of the ring"

    return True, "Molecule is an xylose"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18222',
                          'name': 'xylose',
                          'definition': 'An aldopentose, found in the embryos '
                                        'of most edible plants and used in '
                                        'medicine to test for malabsorption by '
                                        'administration in water to the '
                                        'patient.',
                          'parents': ['CHEBI:33916']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183921,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.999994562912539}


"""
Classifies: CHEBI:25622 orthoquinones
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_orthoquinones(smiles: str):
    """
    Determines if a molecule is an orthoquinone (quinone with carbonyl groups joined by single bond).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an orthoquinone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Look for carbonyl groups (C=O)
    carbonyl_patterns = mol.GetSubstructMatches(Chem.MolFromSmarts('[C;!R]=[O;!R]'))
    ring_carbonyl_patterns = mol.GetSubstructMatches(Chem.MolFromSmarts('[C;R]=[O;!R]'))
    
    all_carbonyls = carbonyl_patterns + ring_carbonyl_patterns
    
    if len(all_carbonyls) < 2:
        return False, "Less than 2 carbonyl groups found"
        
    # Check pairs of carbonyls to find adjacent ones
    for i in range(len(all_carbonyls)):
        for j in range(i+1, len(all_carbonyls)):
            c1 = all_carbonyls[i][0]  # Carbon atom index of first carbonyl
            c2 = all_carbonyls[j][0]  # Carbon atom index of second carbonyl
            
            # Check if carbonyls are connected by single bond
            bond = mol.GetBondBetweenAtoms(c1, c2)
            if bond is not None and bond.GetBondType() == Chem.BondType.SINGLE:
                # Verify both carbonyls are part of the same ring system
                if mol.GetAtomWithIdx(c1).IsInRing() and mol.GetAtomWithIdx(c2).IsInRing():
                    # Additional check to ensure carbons are in same ring
                    ring_info = mol.GetRingInfo()
                    rings = ring_info.AtomRings()
                    for ring in rings:
                        if c1 in ring and c2 in ring:
                            return True, "Found adjacent carbonyl groups in ring system connected by single bond"
                
    return False, "No adjacent carbonyl groups found connected by single bond in ring system"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25622',
                          'name': 'orthoquinones',
                          'definition': 'Any quinone in which the carbons of '
                                        'the two carbonyl groups in the '
                                        'quinone system are joined to each '
                                        'other by a single bond.',
                          'parents': ['CHEBI:36141']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: cannot import name 'rdDecomposition' from "
               "'rdkit.Chem' "
               '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 36779,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 0.75,
    'f1': 0.056074766355140186,
    'accuracy': 0.9972616110403166}


"""
Classifies: CHEBI:26888 tetrachlorobenzene
"""
from rdkit import Chem

def is_tetrachlorobenzene(smiles: str):
    """
    Determines if a molecule is a tetrachlorobenzene (any chlorobenzene carrying four chloro groups).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a tetrachlorobenzene, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Count the number of chlorine atoms
    num_cl = sum(atom.GetSymbol() == 'Cl' for atom in mol.GetAtoms())

    # Check for exactly four chlorine atoms
    if num_cl != 4:
        return False, "The molecule does not contain exactly four chlorine atoms"

    # Check for at least one aromatic ring
    if mol.GetRingInfo().NumAromaticRings() < 1:
        return False, "The molecule does not contain an aromatic ring"

    # Check if all non-chlorine atoms are carbon or hydrogen
    for atom in mol.GetAtoms():
        symbol = atom.GetSymbol()
        if symbol != 'C' and symbol != 'H' and symbol != 'Cl':
            return False, "The molecule contains atoms other than carbon, hydrogen, and chlorine"

    return True, "The molecule is a tetrachlorobenzene"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26888',
                          'name': 'tetrachlorobenzene',
                          'definition': 'Any member of the class of '
                                        'chlorobenzenes carrying four chloro '
                                        'groups at unspecified positions.',
                          'parents': ['CHEBI:23132']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "'RingInfo' object has no attribute 'NumAromaticRings'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:33257 secondary amide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_secondary_amide(smiles: str):
    """
    Determines if a molecule is a secondary amide, defined as a derivative of two oxoacids RkE(=O)l(OH)m (l =/= 0)
    in which two acyl groups are attached to the amino or substituted amino group.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a secondary amide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find all amide bonds
    amide_bonds = []
    for bond in mol.GetBonds():
        begin_atom = mol.GetAtomWithIdx(bond.GetBeginAtomIdx())
        end_atom = mol.GetAtomWithIdx(bond.GetEndAtomIdx())
        if begin_atom.GetAtomicNum() == 7 and end_atom.GetAtomicNum() == 8:
            amide_bonds.append(bond)

    # Check if there are at least two amide bonds
    if len(amide_bonds) < 2:
        return False, "Less than two amide bonds found"

    # Check if the amide bonds are attached to the same nitrogen atom
    nitrogen_atoms = []
    for bond in amide_bonds:
        begin_atom = mol.GetAtomWithIdx(bond.GetBeginAtomIdx())
        end_atom = mol.GetAtomWithIdx(bond.GetEndAtomIdx())
        if begin_atom.GetAtomicNum() == 7:
            nitrogen_atoms.append(begin_atom)
        else:
            nitrogen_atoms.append(end_atom)

    if len(set(nitrogen_atoms)) != 1:
        return False, "Amide bonds are not attached to the same nitrogen atom"

    # Check if the nitrogen atom has at least one hydrogen atom
    hydrogen_count = sum(1 for atom in nitrogen_atoms[0].GetNeighbors() if atom.GetAtomicNum() == 1)
    if hydrogen_count == 0:
        return False, "Nitrogen atom does not have any hydrogen atoms"

    # Check if the acyl groups are attached to the nitrogen atom
    acyl_groups = []
    for bond in amide_bonds:
        begin_atom = mol.GetAtomWithIdx(bond.GetBeginAtomIdx())
        end_atom = mol.GetAtomWithIdx(bond.GetEndAtomIdx())
        if begin_atom.GetAtomicNum() == 8:
            acyl_groups.append(begin_atom)
        else:
            acyl_groups.append(end_atom)

    if len(acyl_groups) != 2:
        return False, "Incorrect number of acyl groups found"

    return True, "Molecule is a secondary amide"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33257',
                          'name': 'secondary amide',
                          'definition': 'A derivative of two oxoacids '
                                        'RkE(=O)l(OH)m (l =/= 0) in which two '
                                        'acyl groups are attached to the amino '
                                        'or substituted amino group.',
                          'parents': ['CHEBI:32988']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183678,
    'num_false_negatives': 25,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.999863910769013}


"""
Classifies: CHEBI:25633 octadecatrienoic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_octadecatrienoic_acid(smiles: str):
    """
    Determines if a molecule is an octadecatrienoic acid (C18 fatty acid with 3 double bonds).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an octadecatrienoic acid, False otherwise 
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if not mol.HasSubstructMatch(carboxylic_pattern):
        return False, "No carboxylic acid group found"
        
    # Count carbon atoms
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count != 18:
        return False, f"Contains {carbon_count} carbons, not 18"

    # Count double bonds (excluding carboxylic acid)
    double_bond_pattern = Chem.MolFromSmarts('C=C')
    double_bonds = len(mol.GetSubstructMatches(double_bond_pattern))
    
    if double_bonds != 3:
        return False, f"Contains {double_bonds} double bonds, not 3"

    # All checks passed
    return True, "Contains 18 carbons and 3 double bonds with carboxylic acid group"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25633',
                          'name': 'octadecatrienoic acid',
                          'definition': 'Any trienoic fatty acid having '
                                        'eighteen carbons.',
                          'parents': [   'CHEBI:132502',
                                         'CHEBI:15904',
                                         'CHEBI:59202']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 72,
    'num_true_negatives': 183795,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.06493506493506493,
    'recall': 0.8333333333333334,
    'f1': 0.12048192771084336,
    'accuracy': 0.9996029868441805}


"""
Classifies: CHEBI:25401 monomethoxyflavone
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_monomethoxyflavone(smiles: str):
    """
    Determines if a molecule is a monomethoxyflavone (flavone with exactly one methoxy substituent).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a monomethoxyflavone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for flavone core structure
    flavone_pattern = Chem.MolFromSmarts('[#6]1=[#6]-c2c([#6](=[O])[#6]1)c(=[#6])c([#6,#8,#1])c([#6,#8,#1])c2')
    if not mol.HasSubstructMatch(flavone_pattern):
        return False, "Not a flavone structure"

    # Count methoxy groups
    methoxy_pattern = Chem.MolFromSmarts('CO')
    methoxy_matches = mol.GetSubstructMatches(methoxy_pattern)
    
    if len(methoxy_matches) == 0:
        return False, "No methoxy groups found"
    elif len(methoxy_matches) > 1:
        return False, f"Contains {len(methoxy_matches)} methoxy groups - more than one"
    
    # Verify the methoxy is attached to the flavone core
    flavone_atoms = set([atom_idx for match in mol.GetSubstructMatches(flavone_pattern) for atom_idx in match])
    methoxy_o = methoxy_matches[0][0]
    methoxy_c = methoxy_matches[0][1]
    
    # Get the atom the methoxy oxygen is bonded to (besides the methyl carbon)
    o_atom = mol.GetAtomWithIdx(methoxy_o)
    connected_atoms = [neighbor.GetIdx() for neighbor in o_atom.GetNeighbors() if neighbor.GetIdx() != methoxy_c]
    
    if not any(atom_idx in flavone_atoms for atom_idx in connected_atoms):
        return False, "Methoxy group not attached to flavone core"

    return True, "Valid monomethoxyflavone structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25401',
                          'name': 'monomethoxyflavone',
                          'definition': 'Any methoxyflavone with a single '
                                        'methoxy substituent.',
                          'parents': ['CHEBI:25241']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183849,
    'num_false_negatives': 8,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.999956487922679}


"""
Classifies: CHEBI:23778 dihydroxybenzoic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_dihydroxybenzoic_acid(smiles: str):
    """
    Determines if a molecule is a dihydroxybenzoic acid or derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a dihydroxybenzoic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Find aromatic rings
    rings = mol.GetRingInfo()
    aromatic_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetIsAromatic() for atom in atoms):
                aromatic_rings.append(ring)
                
    if not aromatic_rings:
        return False, "No aromatic 6-membered rings found"
        
    # For each aromatic ring, check if it has:
    # 1. Two OH groups directly attached
    # 2. A carboxylic acid group (-COOH) attached
    for ring_atoms in aromatic_rings:
        hydroxy_count = 0
        has_carboxyl = False
        
        for atom_idx in ring_atoms:
            atom = mol.GetAtomWithIdx(atom_idx)
            
            # Check neighbors of ring atoms
            for neighbor in atom.GetNeighbors():
                if neighbor.GetIdx() not in ring_atoms:
                    # Check for OH groups
                    if neighbor.GetSymbol() == 'O' and neighbor.GetTotalNumHs() == 1:
                        hydroxy_count += 1
                    
                    # Check for carboxylic acid group
                    elif neighbor.GetSymbol() == 'C':
                        # Look for C(=O)O pattern
                        for n2 in neighbor.GetNeighbors():
                            if n2.GetSymbol() == 'O':
                                if n2.GetTotalNumHs() == 1:  # OH part
                                    for n3 in neighbor.GetNeighbors():
                                        if n3.GetSymbol() == 'O' and n3.GetTotalNumHs() == 0:  # =O part
                                            has_carboxyl = True
                                            
        if hydroxy_count >= 2 and has_carboxyl:
            return True, f"Found aromatic ring with {hydroxy_count} hydroxy groups and carboxylic acid group"
            
    return False, "Structure does not match dihydroxybenzoic acid pattern"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23778',
                          'name': 'dihydroxybenzoic acid',
                          'definition': 'Any member of the class of  '
                                        'hydroxybenzoic acids carrying  two '
                                        'phenolic hydroxy groups on the '
                                        'benzene ring and its derivatives.',
                          'parents': ['CHEBI:24676']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 103609,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9990357914224005}